from typing_extensions import Self

from .. import api


class LoggerSection(api.EnvConfigSection):
    def __init__(self) -> None:
        super().__init__()
        self.logfile = None
        self.log_handler = None
        self.log_request = False
        self.log_response = False
        self.log_web = False
        self.log_sql = False
        self.log_db = False
        self.log_db_level = None
        self.log_level = None

    def init(self, curr_env: api.Env) -> Self:
        self.logfile = curr_env.get("LOGFILE")
        self.log_handler = curr_env.get("LOG_HANDLER")
        self.log_request = curr_env.get_bool("LOG_REQUEST")
        self.log_response = curr_env.get_bool("LOG_RESPONSE")
        self.log_web = curr_env.get_bool("LOG_WEB")
        self.log_sql = curr_env.get_bool("LOG_SQL")
        self.log_db = curr_env.get_bool("LOG_DB")
        self.log_db_level = curr_env.get("LOG_DB_LEVEL")
        self.log_level = curr_env.get("LOG_LEVEL")
        return self

    def to_values(self) -> api.OdooCliFlag:
        flags = api.OdooCliFlag()
        flags.set("logfile", self.logfile)
        flags.set("log-handler", self.log_handler)
        flags.set("log-request", self.log_request)
        flags.set("log-response", self.log_response)
        flags.set("log-web", self.log_web)
        flags.set("log-sql", self.log_sql)
        flags.set("log-db", self.log_db)
        flags.set("log-db-level", self.log_db_level)
        flags.set("log-level", self.log_level)
        return flags



"""Generate the code reference pages and navigation."""

import sys
from pathlib import Path

import mkdocs_gen_files

nav = mkdocs_gen_files.Nav()

config = mkdocs_gen_files.FilesEditor.current()

watched_paths = config.config["plugins"]["mkdocstrings"].config.data["handlers"]["python"]["paths"]
processed_path = []

if sys.version_info >= (3, 9):

    def is_relative_to(path1: Path, path2: Path):
        return path1.is_relative_to(path2)

else:

    def is_relative_to(path1: Path, path2: Path):
        try:
            path1.relative_to(path2)
            return True
        except ValueError:
            return False


for used_path in watched_paths:
    # TODO exclude the other path because handle after
    print(">", used_path)
    for path in sorted(Path(used_path).rglob("*.py")):
        print(">>", path)
        if any(
            is_relative_to(path, watch_path)
            for watch_path in watched_paths
            if watch_path != used_path and watch_path not in processed_path
        ):
            print(">>> Is relative !!!!!")
            continue

        if path.name == "_version.py":
            print("Exclude")
            continue

        module_path = path.relative_to(used_path).with_suffix("")
        doc_path = path.relative_to(used_path).with_suffix(".md")

        full_doc_path = Path("reference", doc_path)

        parts = tuple(module_path.parts)

        if parts[-1] == "__init__":
            parts = parts[:-1]
            doc_path = doc_path.with_name("index.md")
            full_doc_path = full_doc_path.with_name("index.md")
        elif parts[-1] == "__main__":
            continue

        nav[parts] = doc_path.as_posix()

        with mkdocs_gen_files.open(full_doc_path, "w") as fd:
            ident = ".".join(parts)
            fd.write(f"::: {ident}")

        mkdocs_gen_files.set_edit_path(full_doc_path, path)
    processed_path.append(used_path)

with mkdocs_gen_files.open("reference/SUMMARY.md", "w") as nav_file:
    nav_file.writelines(nav.build_literate_nav())



"""
Python file containing all the function used by the cli.
`odoo_env_confi` expose a script command when installed.
>>> odoo - env2config - h

This command allow to run this libraray inside an odoo command (See `odoo.cli.Command`
"""

import argparse
import logging
import os
import subprocess
import sys
from os.path import join as pjoin
from typing import List

from dotenv import dotenv_values

from . import api, entry


def init_logger():
    _logger_level = getattr(logging, os.environ.get("NDP_SERVER_LOG_LEVEL", "INFO"), logging.INFO)
    _logger = logging.getLogger(__name__)
    _logger.setLevel(logging.DEBUG)
    _logger.addHandler(logging.StreamHandler())


class SplitArgs(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        # start or ending comma removed to avoid empty value in list
        setattr(
            namespace,
            self.dest,
            getattr(namespace, self.dest, []) + list(filter(None, values.split(","))),
        )


def get_odoo_cmd_parser() -> argparse.ArgumentParser:
    p = argparse.ArgumentParser()
    p.add_argument("--profile", action="append", dest="profiles", default=[], help="profile to use")
    p.add_argument(
        "--profiles",
        action=SplitArgs,
        dest="profiles",
        default=[],
        help="profiles to use (comma separated)",
    )
    p.add_argument("--dest", dest="config_dest", help="Path to odoo configuration file")
    return p


class OdooCommand:
    """
    This contains a new OdooCommand named `env2config`
    This class prupose is to use it inside an Odoo modules
    This command use `odoo_env_config` to convert the [os.environ][os.environ] to a valid Odoo config
    Args:
        --profiles List[str]: A list of env file to load as extra instead of only looking in [os.environ][os.environ]
        --dest str: The path where we store the result odoo config file.
    Examples:
        >>> python \
/odoo/bin --addons-path=odoo_env_config/odoo_modules env2config \
--profile=/tmp/profile1.env \
--profile=/tmp/profiles2.env \
--dest=/tmp
    """

    def __init__(self, odoo_module):
        self.odoo_module = odoo_module
        type(
            type(self).command_name(),
            (self.odoo_module.cli.Command,),
            {"run": self.run_command},
        )

    @classmethod
    def command_name(cls) -> str:
        return "env2config"

    def run_command(self, args: List[str]):
        """
        Entrypoint of the command

        1. First we parse `args`
        2. Then we load `--profiles` if some are provided
        3. And finaly we execute [odoo_env_config][odoo_env_config.entry.env_to_odoo_args] and save it to the dest file

        Args:
            args: the args provided by Odoo
        """
        ns, sub_args = get_odoo_cmd_parser().parse_known_args(args)
        # Removing blank sub_args
        # Is called with "$ENV_VAR" but ENV_VAR isn't set, then `sub_args` contains `['']
        # So we remove empty string from it
        sub_args = [s for s in sub_args if s.split()]
        extra_env = _load_profiles(ns.profiles)
        entry.direct_run_command(self.odoo_module, sub_args, ns.config_dest, extra_env)


def _load_profiles(profiles: List[str]) -> api.Env:
    extra_env = api.Env()
    for profile in profiles:
        path_profile = os.path.abspath(os.path.expanduser(profile))
        if not os.path.exists(path_profile):
            print("Can't load", path_profile, "file don't exist")
        else:
            extra_env.update(dotenv_values(path_profile))
    return extra_env


load_profiles = _load_profiles


def env_to_config():
    # add_help=False otherwise conflict with -h parameter of parent parser
    parser = argparse.ArgumentParser(parents=[get_odoo_cmd_parser()], add_help=False)
    parser.add_argument(
        "odoo_bin",
        default=api.Env().odoo_bin,
        nargs="?",
        help="Path to odoo-bin file, env:ODOO_BIN or /odoo/odoo-bin is used",
    )
    ns, other = parser.parse_known_args()
    parser.exit(execute_odoo_cli_helper(ns.odoo_bin, ns.profiles, ns.config_dest, other))


def execute_odoo_cli_helper(odoo_bin: str, profiles: List[str], odoo_rc: str, extra_odoo_args: List[str] = None) -> int:
    return execute_odoo_cli(
        os.path.abspath(os.path.expanduser(odoo_bin)),
        ["--profile=" + profile for profile in profiles] + ["--dest=" + odoo_rc] + extra_odoo_args,
    )


def execute_odoo_cli(odoo_bin, other) -> int:
    if not os.path.exists(odoo_bin):
        print("odoo-bin path don't exist", file=sys.stderr)
        return 1
    cmd_args = [sys.executable, odoo_bin]
    addons_path = pjoin(os.path.dirname(__file__), "odoo_modules")
    cmd_args.append(f"--addons-path={addons_path}")
    cmd_args.append(OdooCommand.command_name())  # The name of the class in odoo_modules/env2config/cli/env2config
    cmd_args.extend(other)
    print(" ".join(cmd_args))
    p = subprocess.Popen(cmd_args)
    return p.wait()



import unittest

from src.odoo_env_config.api import Env, OdooConfig
from src.odoo_env_config.section.odoo_opt_section import OdooOptSection
from typing import cast


class TestOdooOptSection(unittest.TestCase):
    def _prepare_conf(self):
        return OdooOptSection().init(
            Env(
                {
                    "OPT_ODOO_MODULES_AUTO_INSTALL_DISABLED": "partner_autocomplete,iap,mail_bot",
                    "OPT_ODOO_MODULES_AUTO_INSTALL_ENABLED": "web_responsive",
                }
            )
        )

    def test_default(self):
        conf = OdooOptSection()
        self.assertEqual([], conf.options)

    def test_global(self):
        conf = self._prepare_conf()
        self.assertTrue(conf.options)
        self.assertListEqual(["modules_auto_install_disabled", "modules_auto_install_enabled"], [ list(item.keys())[0] for item in conf.options])
        self.assertIn({"modules_auto_install_disabled": "partner_autocomplete,iap,mail_bot"}, conf.options)
        self.assertIn({"modules_auto_install_enabled": "web_responsive"}, conf.options)

    def test_write_to_config(self):
        conf = self._prepare_conf()
        config: OdooConfig = {"db_port": "5469"} # type: ignore
        conf.write_to_config(config)
        self.assertEqual("partner_autocomplete,iap,mail_bot", config["modules_auto_install_disabled"])
        self.assertEqual("web_responsive", config["modules_auto_install_enabled"])
        self.assertEqual("5469", config["db_port"], "pas de changements")



import unittest

from src.odoo_env_config.api import Env
from src.odoo_env_config.section import TestOdooConfigSection
from tests._decorators import MultiOdooVersion


class TestTestSection(unittest.TestCase):
    @MultiOdooVersion.without_args
    def test_default(self):
        conf = TestOdooConfigSection()
        self.assertFalse(conf.enable)
        self.assertIsNone(conf.test_tags)
        self.assertIsNone(conf.test_file)

        flags = conf.to_values()
        self.assertFalse(flags)

    @MultiOdooVersion.with_args
    def test_version_disable(self, version: int):
        conf = TestOdooConfigSection().init(
            Env(
                {
                    "ODOO_VERSION": str(version),
                    "TEST_ENABLE": str(False),
                    "TEST_TAGS": "test_tags_value",
                    "TEST_FILE": "test_file_value",
                }
            )
        )
        flags = conf.to_values()
        self.assertFalse(conf.enable)
        self.assertFalse(flags)

    @MultiOdooVersion.with_args
    def test_odoo_test_tags(self, version: int):
        conf = TestOdooConfigSection().init(
            Env(
                {
                    "ODOO_VERSION": str(version),
                    "TEST_ENABLE": str(True),
                    "TEST_TAGS": "test_tags_value",
                    "TEST_FILE": "test_file_value",
                }
            )
        )
        flags = conf.to_values()

        self.assertTrue(conf.enable)
        self.assertIn("test-enable", flags)
        self.assertTrue(flags["test-enable"])

        # test-tags not supported in Odoo 11 or less
        if version > 11:
            self.assertIn("test-tags", flags)
            self.assertEqual("test_tags_value", flags["test-tags"])
        else:
            self.assertNotIn("test-tags", flags)

        self.assertEqual("test_file_value", conf.test_file)
        self.assertIn("test-file", flags)
        self.assertEqual("test_file_value", flags["test-file"])



class ConfigWriter:
    def __init__(self):
        self.misc = {}
        self.options = {}

    def get(self, key, default=None):
        return self.options.get(key, default)

    def pop(self, key, default=None):
        return self.options.pop(key, default)

    def get_misc(self, sect, key, default=None):
        return self.misc.get(sect, {}).get(key, default)

    def __setitem__(self, key, value):
        self.options[key] = value

    def __getitem__(self, key):
        return self.options[key]



import unittest

from tests import _decorators

from ._helpers import assertParser, create_config


@_decorators.SkipUnless.env_odoo
class TestOdooOpt(unittest.TestCase):
    def test_odoo_opt(self):
        """
        Assert module `odoo_filestore_s3` is enable.
        Returns:

        """
        parser = create_config(["odoo_opt"])
        assertParser(
            self,
            parser,
            {
                "modules_auto_install_enabled": "web_responsive",
                "modules_auto_install_disabled": "partner_autocomplete,iap,mail_bot",
            },
        )



"""
Contains the mapper specifique for the environment variable provided by CleverCloud addons.
Currently we support :
- S3 addons Cellar
- Postgres Addons of any scaler
"""

from . import utils
from .api import Env

__all__ = [
    "clevercloud_cellar",
    "clevercloud_postgresql",
    "compatibility",
    "queue_job",
    "redis_session",
]


def clevercloud_cellar(curr_env: Env) -> Env:
    """ """
    return curr_env.copy().mutate(
        {
            "S3_FILESTORE_HOST": curr_env.gets(
                "S3_FILESTORE_HOST", "CELLAR_ADDON_HOST", "ODOO_S3_HOST", "MINIO_DOMAIN"
            ),
            "S3_FILESTORE_SECRET_KEY": curr_env.gets(
                "S3_FILESTORE_SECRET_KEY",
                "CELLAR_ADDON_KEY_SECRET",
                "ODOO_S3_SECRET_KEY",
                "MINIO_SECRET_KEY",
            ),
            "S3_FILESTORE_ACCESS_KEY": curr_env.gets(
                "S3_FILESTORE_ACCESS_KEY",
                "CELLAR_ADDON_KEY_ID",
                "ODOO_S3_ACCESS_KEY",
                "MINIO_ACCESS_KEY",
            ),
            "S3_FILESTORE_BUCKET": curr_env.gets("S3_FILESTORE_BUCKET", "ODOO_S3_BUCKET"),
            # Pas de region fournit par S3 CleverCloud
            "S3_FILESTORE_REGION": curr_env.gets("S3_FILESTORE_REGION", "CELLAR_ADDON_REGION", "ODOO_S3_REGION"),
            "S3_SECURE": curr_env.gets("S3_FILESTORE_SECURE", "ODOO_S3_SECURE"),
        },
    )


def clevercloud_postgresql(curr_env: Env) -> Env:
    """ """
    return curr_env.copy().mutate(
        {
            "DB_NAME": utils.get_value(curr_env, "DB_NAME", "DATABASE", "POSTGRESQL_ADDON_DB", "POSTGRES_DB"),
            "DB_HOST": utils.get_value(
                curr_env,
                "DB_HOST",
                "POSTGRESQL_ADDON_DIRECT_HOST",
                "POSTGRESQL_ADDON_HOST",
            ),
            "DB_PORT": utils.get_value(
                curr_env,
                "DB_PORT",
                "POSTGRESQL_ADDON_DIRECT_PORT",
                "POSTGRESQL_ADDON_PORT",
            ),
            "DB_USER": utils.get_value(curr_env, "DB_USER", "POSTGRESQL_ADDON_USER", "POSTGRES_USER"),
            "DB_PASSWORD": utils.get_value(
                curr_env,
                "DB_PASSWORD",
                "POSTGRESQL_ADDON_PASSWORD",
                "POSTGRES_PASSWORD",
            ),
        }
    )


def compatibility(curr_env: Env) -> Env:
    """ """
    return curr_env.copy().mutate(
        {
            "PROXY_MODE": utils.get_value(curr_env, "PROXY_MODE", "PROXY_ENABLE"),
            "WORKER_HTTP": utils.get_value(curr_env, "WORKER_HTTP", "WORKERS"),
            "WORKER_CRON": utils.get_value(curr_env, "WORKER_CRON", "CRON_THREAD", "MAX_CRON_THREAD"),
            "WORKER_JOB": utils.get_value(curr_env, "WORKER_JOB"),
            "HTTP_INTERFACE": utils.get_value(curr_env, "HTTP_INTERFACE", "XMLRPC_INTERFACE"),
            "HTTP_PORT": utils.get_value(curr_env, "HTTP_PORT", "XMLRPC_PORT"),
            "HTTP_ENABLE": utils.get_value(curr_env, "HTTP_ENABLE", "XMLRPC_ENABLE"),
            "LONGPOLLING_PORT": utils.get_value(curr_env, "GEVENT_PORT", "LONGPOLLING_PORT"),
            "SERVER_WIDE_MODULES": utils.get_value(curr_env, "SERVER_WIDE_MODULES", "LOAD"),
        }
    )


def queue_job(curr_env: Env) -> Env:
    """ """
    new_env = curr_env.copy()
    enable = utils.to_bool(curr_env.get("ODOO_QUEUE_JOB_ENABLE"))
    if not enable:
        return new_env.mutate(ODOO_QUEUE_JOB_ENABLE=str(False))

    def copy(s):
        return [p + s for p in ["ODOO_QUEUE_JOB_", "ODOO_CONNECTOR_"]]

    return new_env.mutate(
        ODOO_QUEUE_JOB_ENABLE=str(True),
        **{
            "ODOO_QUEUE_JOB_CHANNELS": utils.get_value(curr_env, *copy("CHANNELS")),
            "ODOO_QUEUE_JOB_SCHEME": utils.get_value(curr_env, *copy("SCHEME")),
            "ODOO_QUEUE_JOB_HOST": utils.get_value(curr_env, *copy("HOST")),
            "ODOO_QUEUE_JOB_PORT": utils.get_value(curr_env, *copy("PORT")),
            "ODOO_QUEUE_JOB_HTTP_AUTH_USER": utils.get_value(curr_env, *copy("HTTP_AUTH_USER")),
            "ODOO_QUEUE_JOB_HTTP_AUTH_PASSWORD": utils.get_value(curr_env, *copy("HTTP_AUTH_PASSWORD")),
            "ODOO_QUEUE_JOB_JOBRUNNER_DB_HOST": utils.get_value(curr_env, *copy("JOBRUNNER_DB_HOST")),
            "ODOO_QUEUE_JOB_JOBRUNNER_DB_PORT": utils.get_value(curr_env, *copy("JOBRUNNER_DB_PORT")),
        },
    )


def redis_session(curr_env: Env) -> Env:
    """ """
    new_env = curr_env.copy()
    enable = curr_env.get_bool(
        "REDIS_SESSION_ENABLE",
        default=bool(new_env.gets("REDIS_SESSION_HOST", "REDIS_HOST")),
    )
    if not enable:
        return new_env.mutate(REDIS_SESSION_ENABLE=str(False))
    return new_env.mutate(
        REDIS_SESSION_ENABLE=str(True),
        **{
            "REDIS_SESSION_URL": utils.get_value(curr_env, "REDIS_SESSION_URL", "REDIS_URL"),
            "REDIS_SESSION_HOST": utils.get_value(curr_env, "REDIS_SESSION_HOST", "REDIS_HOST"),
            "REDIS_SESSION_PORT": utils.get_value(curr_env, "REDIS_SESSION_PORT", "REDIS_PORT"),
            "REDIS_SESSION_DB_INDEX": utils.get_value(curr_env, "REDIS_SESSION_DB_INDEX", "REDIS_DB_INDEX"),
            "REDIS_SESSION_PASSWORD": utils.get_value(curr_env, "REDIS_SESSION_PASSWORD", "REDIS_PASSWORD"),
        },
    )



__all__ = [
    "AddonsPathConfigSection",
    "DatabaseOdooConfigSection",
    "HttpOdooConfigSection",
    "LimitOdooConfigSection",
    "LoggerSection",
    "MiscSection",
    "TestOdooConfigSection",
    "UpdateInstallSection",
    "ServerWideModuleConfigSection",
    "WorkersOdooConfigSection",
    "S3Section",
    "OdooOptSection",
]
from .addons_path_section import AddonsPathConfigSection
from .db_section import DatabaseOdooConfigSection
from .http_section import HttpOdooConfigSection
from .limit_section import LimitOdooConfigSection
from .log_section import LoggerSection
from .misc_section import MiscSection
from .odoo_opt_section import OdooOptSection
from .s3_section import S3Section
from .test_section import TestOdooConfigSection
from .update_init_section import UpdateInstallSection
from .widemodule_section import ServerWideModuleConfigSection
from .worker_section import WorkersOdooConfigSection



import os
from typing import List

from typing_extensions import Self

from .. import api


class MiscSection(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.unaccent = False
        self.without_demo: List[str] = []
        self.stop_after_init = False
        self.admin_password = False
        self.save_config_file = False
        self.data_dir = None

    def init(self, curr_env: api.Env) -> Self:
        self.unaccent = curr_env.get_bool("UNACCENT", default=False)
        self.without_demo = curr_env.get_list("WITHOUT_DEMO")
        if curr_env.is_boolean("WITHOUT_DEMO") and curr_env.get_bool("WITHOUT_DEMO"):
            # When WITHOUT_DEMO=True, then we set to `all`
            self.without_demo = ["all"]
        self.stop_after_init = curr_env.get_bool("STOP_AFTER_INIT")
        self.save_config_file = curr_env.get_bool("SAVE_CONFIG_FILE")
        self.data_dir = curr_env.get("DATA_DIR")
        if curr_env.get("ODOO_PATH"):
            self.data_dir = os.path.join(curr_env.get("ODOO_PATH"), self.data_dir)

        self.admin_password = curr_env.gets("ADMIN_PASSWORD", "ADMIN_PASSWD", "ODOO_ADMIN_PASSWORD")
        return self

    def to_values(self) -> api.OdooCliFlag:
        flags = api.OdooCliFlag()
        flags.set("unaccent", self.unaccent)
        flags.set("without-demo", ",".join(self.without_demo))
        flags.set("save", self.save_config_file)
        flags.set("stop-after-init", self.stop_after_init)
        flags.set("data-dir", self.data_dir)
        return flags



from typing_extensions import Self

from .. import api
from .test_section import TestOdooConfigSection


class WorkersOdooConfigSection(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.http: int = 0
        self.cron: int = 0
        self.job: int = 0
        self.odoo_use_case: api.ServerUseCase = api.ServerUseCase.WORKER
        self._force_set_cron: bool = False

    def init(self, env: api.Env) -> Self:
        self.odoo_use_case = env.get_enum("ODOO_USE_CASE", api.ServerUseCase, default=api.ServerUseCase.WORKER)
        if TestOdooConfigSection().init(env).enable:
            self.odoo_use_case = api.ServerUseCase.TESTS

        if self.odoo_use_case == api.ServerUseCase.WORKER:
            self.cron = env.get_int("WORKER_CRON")
            self.job = env.get_int("WORKER_JOB")
            self.http = env.get_int("WORKER_HTTP")
        elif self.odoo_use_case == api.ServerUseCase.THREADED_MODE:
            self.cron = env.get_int("WORKER_CRON")
        elif self.odoo_use_case == api.ServerUseCase.ONLY_JOB_WORKER:
            self.job = env.get_int("WORKER_JOB")
        elif self.odoo_use_case == api.ServerUseCase.ONLY_HTTP:
            self.http = env.get_int("WORKER_HTTP")
        elif self.odoo_use_case == api.ServerUseCase.ONLY_CRON:
            self.cron = env.get_int("WORKER_CRON")
        else:
            # api.ServerUseCase.TESTS
            # api.ServerUseCase.ONLY_JOB_RUNNER
            self.http = self.cron = self.job = 0

        self._force_set_cron = not env.main_instance and self.odoo_use_case != api.ServerUseCase.ONLY_CRON
        # In case not mail instance, then we don't want to have cron again, only on the main instance
        if self._force_set_cron:
            self.cron = 0

        return self

    @property
    def total(self):
        return self.worker + self.cron

    @property
    def worker(self):
        return self.http + self.job

    def to_values(self) -> api.OdooCliFlag:
        res = api.OdooCliFlag()
        res.set("worker", self.worker)
        if self.odoo_use_case in {
            api.ServerUseCase.ONLY_CRON,
            api.ServerUseCase.WORKER,
            api.ServerUseCase.THREADED_MODE,
        }:
            res.set("max-cron-thread", self.cron, force_set=self._force_set_cron)
        return res



"""Generate the code reference pages and navigation."""

from pathlib import Path

from mkdocs_gen_files import FilesEditor

config: FilesEditor = FilesEditor.current()

index_md_path = Path("index.md")
print(index_md_path)
readme_md_path = Path(config.config.config_file_path).parent.joinpath("README.md")
print(readme_md_path)
with open(str(readme_md_path), "r") as f_src, config.open(str(index_md_path), "w") as f_dest:
    f_dest.writelines(f_src.readlines())



import unittest

from src.odoo_env_config import utils


class TestUtils(unittest.TestCase):
    def test_is_true(self):
        self.assertTrue(utils.to_bool(str(True)))
        self.assertTrue(utils.to_bool(str(1)))

        self.assertTrue(utils.to_bool(True))
        self.assertTrue(utils.to_bool(1))

        self.assertFalse(utils.to_bool(str(False)))
        self.assertFalse(utils.to_bool(str(0)))
        self.assertFalse(utils.to_bool(str(0.0)))
        self.assertFalse(utils.to_bool(str(None)))

        self.assertFalse(utils.to_bool(False))
        self.assertFalse(utils.to_bool(0))
        self.assertFalse(utils.to_bool(0.0))
        self.assertFalse(utils.to_bool(1.0))

        self.assertFalse(utils.to_bool(object()))
        self.assertFalse(utils.to_bool(None))
        self.assertFalse(utils.to_bool([]))
        self.assertFalse(utils.to_bool([1, 2, 3]))
        self.assertFalse(utils.to_bool((1, 2, 3)))
        self.assertFalse(utils.to_bool({1, 2, 3}))
        self.assertFalse(utils.to_bool({"key": "value"}))

    def test_to_int(self):
        self.assertEqual(0, utils.to_int(str(True)))
        self.assertEqual(0, utils.to_int(True))
        self.assertEqual(1, utils.to_int(str(1)))

        self.assertEqual(1, utils.to_int(1))

        self.assertEqual(0, utils.to_int(str(False)))
        self.assertEqual(0, utils.to_int(str(0)))
        self.assertEqual(0, utils.to_int(str(0.0)))
        self.assertEqual(0, utils.to_int(str(None)))

        self.assertEqual(0, utils.to_int(False))
        self.assertEqual(0, utils.to_int(0))
        self.assertEqual(0, utils.to_int(0.0))
        self.assertEqual(1, utils.to_int(1.0))
        self.assertEqual(0, utils.to_int(object()))
        self.assertEqual(0, utils.to_int(None))

        self.assertEqual(12, utils.to_int(12.5))
        self.assertEqual(13, utils.to_int(str(13.6)))

    def test_add_dash(self):
        self.assertEqual("--my_option", utils.add_dash("my_option"))
        self.assertEqual("-k=value", utils.add_dash("k=value"))
        self.assertEqual("--key=value", utils.add_dash("key=value"))
        self.assertEqual("--key=value", utils.add_dash("--key=value"))
        self.assertEqual("-o", utils.add_dash("-o"))
        self.assertEqual("-i", utils.add_dash("i"))
        self.assertEqual("-u=base", utils.add_dash("-u=base"))
        self.assertEqual("", utils.add_dash(None))
        self.assertEqual("", utils.add_dash(""))



import unittest

from src.odoo_env_config import api
from src.odoo_env_config.mappers import clevercloud_postgresql


class TestCleverCloudMapperPostgres(unittest.TestCase):
    def _test_priority(self, key, to_map, expected, nb_key=6):
        result = clevercloud_postgresql(api.Env(to_map))
        self.assertEqual(nb_key, len(result.keys()))
        self.assertEqual(
            expected,
            result[key],
            "Value of key [%s] is not '%s' result : '%s'"
            % (key, expected, result[key]),
        )

    def test_no_value(self):
        result = clevercloud_postgresql(api.Env({}))
        self.assertEqual(5, len(result.keys()))
        self.assertIsNone(result["DB_NAME"])
        self.assertIsNone(result["DB_HOST"])
        self.assertIsNone(result["DB_PORT"])
        self.assertIsNone(result["DB_USER"])
        self.assertIsNone(result["DB_PASSWORD"])

    def test_postgres_addon(self):
        db_host = "my-host.com"
        db_name = "my-db"
        db_port = "my-port"
        db_user = "my-user"
        db_password = "py-password"

        result = clevercloud_postgresql(
            api.Env(
                {
                    "POSTGRESQL_ADDON_DB": db_name,
                    "POSTGRESQL_ADDON_HOST": db_host,
                    "POSTGRESQL_ADDON_PORT": db_port,
                    "POSTGRESQL_ADDON_USER": db_user,
                    "POSTGRESQL_ADDON_PASSWORD": db_password,
                }
            )
        )
        self.assertEqual(10, len(result.keys()))
        self.assertEqual(db_name, result["DB_NAME"])
        self.assertEqual(db_host, result["DB_HOST"])
        self.assertEqual(db_port, result["DB_PORT"])
        self.assertEqual(db_user, result["DB_USER"])
        self.assertEqual(db_password, result["DB_PASSWORD"])

    def test_postgres_addon_direct(self):
        """
        Chez Clever cloud les connections sont géré via un proxy,
        il reste possible via une option chez eux d'avoir un acces direct
        Dans ce cas les variables <POSTGRESQL_ADDON_DIRECT_PORT> et <POSTGRESQL_ADDON_DIRECT_HOST> sont disponibles.
        Les variables <POSTGRESQL_ADDON_DIRECT_DB>, <POSTGRESQL_ADDON_DIRECT_USER> et
         <POSTGRESQL_ADDON_DIRECT_PASSWORD> n'existent pas
        """

        db_host = "my-host.com"
        db_name = "my-db"
        db_port = "my-port"
        db_user = "my-user"
        db_password = "py-password"

        result = clevercloud_postgresql(
            api.Env(
                {
                    "POSTGRESQL_ADDON_DIRECT_DB": db_name,
                    "POSTGRESQL_ADDON_DIRECT_HOST": db_host,
                    "POSTGRESQL_ADDON_DIRECT_PORT": db_port,
                    "POSTGRESQL_ADDON_DIRECT_USER": db_user,
                    "POSTGRESQL_ADDON_DIRECT_PASSWORD": db_password,
                }
            )
        )
        self.assertEqual(10, len(result.keys()))
        self.assertIsNone(
            result["DB_NAME"],
            "POSTGRESQL_ADDON_DIRECT_DB n'est pas prise en compte par le mapper",
        )
        self.assertEqual(db_host, result["DB_HOST"])
        self.assertEqual(db_port, result["DB_PORT"])
        self.assertIsNone(
            result["DB_USER"],
            "POSTGRESQL_ADDON_DIRECT_USER n'est pas prise en compte par le mapper",
        )
        self.assertIsNone(
            result["DB_PASSWORD"],
            "POSTGRESQL_ADDON_DIRECT_PASSWORD n'est pas prise en compte par le mapper",
        )

    def test_postgres_addon_direct_priority(self):
        """
        Chez Clever cloud les connections sont géré via un proxy,
        il reste possible via une option chez eux d'avoir un acces direct
        Dans ce cas les variables <POSTGRESQL_ADDON_DIRECT_PORT> et <POSTGRESQL_ADDON_DIRECT_HOST> sont disponibles.
        Dans ce cas, elles doivent être prises en priorité par rapport
         à <POSTGRESQL_ADDON_PORT> et <POSTGRESQL_ADDON_HOST>
        """

        db_host = "my-host.com"
        db_port = "my-port"
        suffix = "_no_take"
        result = clevercloud_postgresql(
            api.Env(
                {
                    "POSTGRESQL_ADDON_DIRECT_HOST": db_host,
                    "POSTGRESQL_ADDON_HOST": db_host + suffix,
                    "POSTGRESQL_ADDON_DIRECT_PORT": db_port,
                    "POSTGRESQL_ADDON_PORT": db_port + suffix,
                }
            )
        )
        self.assertEqual(9, len(result.keys()))
        self.assertIsNone(result["DB_NAME"])
        self.assertEqual(db_host, result["DB_HOST"])
        self.assertEqual(db_port, result["DB_PORT"])
        self.assertIsNone(result["DB_USER"])
        self.assertIsNone(result["DB_PASSWORD"])

    def test_postgres_priority_global(self):
        db_host = "my-host.com"
        db_name = "my-db"
        db_port = "my-port"
        db_user = "my-user"
        db_password = "py-password"
        origin = "_no_take"
        result = clevercloud_postgresql(
            api.Env(
                {
                    "DB_NAME": db_name,
                    "POSTGRESQL_ADDON_DB": db_name + origin,
                    "DB_HOST": db_host,
                    "POSTGRESQL_ADDON_HOST": db_host + origin,
                    "DB_PORT": db_port,
                    "POSTGRESQL_ADDON_PORT": db_port + origin,
                    "DB_USER": db_user,
                    "POSTGRESQL_ADDON_USER": db_user + origin,
                    "DB_PASSWORD": db_password,
                    "POSTGRESQL_ADDON_PASSWORD": db_password + origin,
                }
            )
        )
        self.assertEqual(10, len(result.keys()))
        self.assertEqual(
            db_name,
            result["DB_NAME"],
            "DB_NAME est prioritaire sur POSTGRESQL_ADDON_DB",
        )
        self.assertEqual(
            db_host,
            result["DB_HOST"],
            "DB_HOST est prioritaire sur POSTGRESQL_ADDON_HOST",
        )
        self.assertEqual(
            db_port,
            result["DB_PORT"],
            "DB_PORT est prioritaire sur POSTGRESQL_ADDON_PORT",
        )
        self.assertEqual(
            db_user,
            result["DB_USER"],
            "DB_USER est prioritaire sur POSTGRESQL_ADDON_USER",
        )
        self.assertEqual(
            db_password,
            result["DB_PASSWORD"],
            "DB_PASSWORD est prioritaire sur POSTGRESQL_ADDON_PASSWORD",
        )

    def test_postgres_priority_db_name(self):
        """
        Test de la prise en compte des variables permettant de préciser le nom de la DB
        DB_NAME > POSTGRESQL_ADDON_DB > DATABASE
        """

        db_name = "my-db"
        db_name1 = "my-db1"
        db_name2 = "my-db2"
        self._test_priority(
            "DB_NAME",
            {
                "DATABASE": db_name,
                "DB_NAME": db_name1,
                "POSTGRESQL_ADDON_DB": db_name2,
            },
            db_name1,
            nb_key=7,
        )
        db_name2 = "my-db2"
        self._test_priority(
            "DB_NAME",
            {
                "DATABASE": db_name,
                # "DB_NAME": db_name1,
                "POSTGRESQL_ADDON_DB": db_name2,
            },
            db_name,
            nb_key=7,
        )
        self._test_priority(
            "DB_NAME",
            {
                # "DATABASE": db_name,
                # "DB_NAME": db_name1,
                "POSTGRESQL_ADDON_DB": db_name2,
            },
            db_name2,
        )

    def test_postgres_priority_db_host(self):
        """
        Test de la prise en compte des variables permetant de préciser l'url de la DB
        DB_HOST > POSTGRESQL_ADDON_DIRECT_HOST > POSTGRESQL_ADDON_HOST
        """

        db_value = "value"
        db_value1 = "value1"
        db_value2 = "value2"
        self._test_priority(
            "DB_HOST",
            {
                "DB_HOST": db_value,
                "POSTGRESQL_ADDON_DIRECT_HOST": db_value1,
                "POSTGRESQL_ADDON_HOST": db_value2,
            },
            db_value,
            nb_key=7,
        )
        self._test_priority(
            "DB_HOST",
            {
                # "DB_HOST": db_value,
                "POSTGRESQL_ADDON_DIRECT_HOST": db_value1,
                "POSTGRESQL_ADDON_HOST": db_value2,
            },
            db_value1,
            nb_key=7,
        )
        self._test_priority(
            "DB_HOST",
            {
                # "DB_HOST": db_value,
                # "POSTGRESQL_ADDON_DIRECT_HOST": db_value1,
                "POSTGRESQL_ADDON_HOST": db_value2,
            },
            db_value2,
        )

    def test_postgres_priority_db_port(self):
        """
        Test de la prise en compte des variables permettant de préciser le port de la DB
        DB_PORT > POSTGRESQL_ADDON_DIRECT_PORT > POSTGRESQL_ADDON_PORT
        """

        db_value = "value"
        db_value1 = "value1"
        db_value2 = "value2"
        self._test_priority(
            "DB_PORT",
            {
                "DB_PORT": db_value,
                "POSTGRESQL_ADDON_DIRECT_PORT": db_value1,
                "POSTGRESQL_ADDON_PORT": db_value2,
            },
            db_value,
            nb_key=7,
        )
        self._test_priority(
            "DB_PORT",
            {
                # "DB_PORT": db_value,
                "POSTGRESQL_ADDON_DIRECT_PORT": db_value1,
                "POSTGRESQL_ADDON_PORT": db_value2,
            },
            db_value1,
            nb_key=7,
        )
        self._test_priority(
            "DB_PORT",
            {
                # "DB_PORT": db_value,
                # "POSTGRESQL_ADDON_DIRECT_PORT": db_value1,
                "POSTGRESQL_ADDON_PORT": db_value2,
            },
            db_value2,
        )

    def test_postgres_priority_db_user(self):
        """
        Test de la prise en compte des variables permettant de préciser le nom de l'utilisateur postgres
        DB_USER > POSTGRESQL_ADDON_USER
        """

        db_value = "value"
        db_value1 = "value1"
        self._test_priority(
            "DB_USER",
            {
                "DB_USER": db_value,
                "POSTGRESQL_ADDON_USER": db_value1,
            },
            db_value,
        )
        self._test_priority(
            "DB_USER",
            {
                # "DB_USER": db_value,
                "POSTGRESQL_ADDON_USER": db_value1,
            },
            db_value1,
        )

    def test_postgres_priority_db_password(self):
        """
        Test de la prise en compte des variables permettant de préciser le nom de l'utilisateur postgres
        DB_PASSWORD > POSTGRESQL_ADDON_PASSWORD
        """

        db_value = "value"
        db_value1 = "value1"
        self._test_priority(
            "DB_PASSWORD",
            {
                "DB_PASSWORD": db_value,
                "POSTGRESQL_ADDON_PASSWORD": db_value1,
            },
            db_value,
        )
        self._test_priority(
            "DB_PASSWORD",
            {
                # "DB_PASSWORD": db_value,
                "POSTGRESQL_ADDON_PASSWORD": db_value1,
            },
            db_value1,
        )



import unittest

from src.odoo_env_config.api import Env, OdooCliFlag
from src.odoo_env_config.section.http_section import HttpOdooConfigSection


class TestHttpOdooConfigSection(unittest.TestCase):
    def test_default(self):
        conf = HttpOdooConfigSection()
        self.assertEqual(OdooCliFlag({"no-http": True}), conf.to_values())
        self.assertIsNone(conf.interface)
        self.assertEqual(0, conf.port)
        self.assertFalse(conf.enable)
        self.assertEqual(0, conf.longpolling_port)

    def test_disable(self):
        conf = HttpOdooConfigSection()
        conf.enable = True
        self.assertEqual(OdooCliFlag(), conf.to_values())
        self.assertIsNone(conf.interface)
        self.assertEqual(0, conf.port)
        self.assertTrue(conf.enable)
        self.assertEqual(0, conf.longpolling_port)

    def test_global_http_key(self):
        conf = HttpOdooConfigSection().init(
            Env(
                {
                    "LONGPOLLING_PORT": "4040",
                    "HTTP_INTERFACE": "0.1.2.3",
                    "HTTP_PORT": "8080",
                    "HTTP_ENABLE": "True",
                }
            )
        )
        self.assertEqual("0.1.2.3", conf.interface)
        self.assertEqual(8080, conf.port)
        self.assertTrue(conf.enable)
        self.assertEqual(4040, conf.longpolling_port)
        self.assertDictEqual(
            OdooCliFlag(
                {
                    "longpolling-port": 4040,
                    "http-port": 8080,
                    "http-interface": "0.1.2.3",
                }
            ),
            conf.to_values(),
        )

    def test_enable(self):
        conf = HttpOdooConfigSection().init(
            Env(
                {
                    "HTTP_ENABLE": "True",
                }
            )
        )
        self.assertEqual(OdooCliFlag(), conf.to_values())
        self.assertIsNone(conf.interface)
        self.assertEqual(0, conf.port)
        self.assertTrue(conf.enable)
        self.assertEqual(0, conf.longpolling_port)
        conf = HttpOdooConfigSection().init(
            Env(
                {
                    "HTTP_ENABLE": "False",
                }
            )
        )
        self.assertEqual(OdooCliFlag({"no-http": True}), conf.to_values())
        self.assertIsNone(conf.interface)
        self.assertEqual(0, conf.port)
        self.assertFalse(conf.enable)
        self.assertEqual(0, conf.longpolling_port)

    def test_longpolling_port_before_v16(self):
        for odoo_version in range(11, 16):
            with self.subTest(odoo_version):
                conf = HttpOdooConfigSection().init(
                    Env(
                        {
                            "ODOO_VERSION": odoo_version,
                            "LONGPOLLING_PORT": "4040",
                            "HTTP_INTERFACE": "0.1.2.3",
                            "HTTP_PORT": "8080",
                            "HTTP_ENABLE": "True",
                        }
                    )
                )
                self.assertEqual("0.1.2.3", conf.interface)
                self.assertEqual(8080, conf.port)
                self.assertTrue(conf.enable)
                self.assertEqual(4040, conf.longpolling_port)
                self.assertDictEqual(
                    OdooCliFlag(
                        {
                            "longpolling-port": 4040,
                            "http-port": 8080,
                            "http-interface": "0.1.2.3",
                        }
                    ),
                    conf.to_values(),
                )

    def test_gevent_key_v18(self):
        for odoo_version in range(16, 30):
            with self.subTest(odoo_version):
                conf = HttpOdooConfigSection().init(
                    Env(
                        {
                            "ODOO_VERSION": odoo_version,
                            "LONGPOLLING_PORT": "4040",
                            "HTTP_INTERFACE": "0.1.2.3",
                            "HTTP_PORT": "8080",
                            "HTTP_ENABLE": "True",
                        }
                    )
                )
                self.assertEqual("0.1.2.3", conf.interface)
                self.assertEqual(8080, conf.port)
                self.assertTrue(conf.enable)
                self.assertEqual(4040, conf.longpolling_port)
                self.assertDictEqual(
                    OdooCliFlag(
                        {
                            "gevent-port": 4040,
                            "http-port": 8080,
                            "http-interface": "0.1.2.3",
                        }
                    ),
                    conf.to_values(),
                )




"""
Odoo Commande
"""

import logging

try:
    import odoo

    from odoo_env_config import cli

    cli.OdooCommand(odoo)
except ImportError:
    logging.error("Can't import Odoo, are you sure you use this module the right way ????")



""" """

import contextlib
import os
from typing import Any, Callable, Dict, List, Set, Type, TypeVar

from . import api, mappers
from .section import (
    AddonsPathConfigSection,
    DatabaseOdooConfigSection,
    HttpOdooConfigSection,
    LimitOdooConfigSection,
    LoggerSection,
    MiscSection,
    S3Section,
    ServerWideModuleConfigSection,
    TestOdooConfigSection,
    UpdateInstallSection,
    WorkersOdooConfigSection,
    OdooOptSection,
)

Section = TypeVar("Section", bound=api.EnvConfigSection)

CONVERTER: Set[Type[Section]] = {
    AddonsPathConfigSection,
    DatabaseOdooConfigSection,
    HttpOdooConfigSection,
    LimitOdooConfigSection,
    LoggerSection,
    MiscSection,
    TestOdooConfigSection,
    UpdateInstallSection,
    ServerWideModuleConfigSection,
    WorkersOdooConfigSection,
    S3Section,
    OdooOptSection,
}
MAPPER: Set[Callable[[api.Env], api.Env]] = {
    mappers.compatibility,
    mappers.clevercloud_postgresql,
    mappers.redis_session,
    mappers.clevercloud_cellar,
    mappers.queue_job,
}


def apply_mapper(env: api.Env) -> api.Env:
    """
    Apply the MAPPER on `env` and return a new `api.Env` without mutate `env`
    Args:
        env: The env to map

    Returns:
        A new `api.Env` with all MAPPER applied on.
    """
    curr_env = env.copy()
    for mapper in MAPPER:
        curr_env = mapper(curr_env)
    return curr_env


def apply_converter(env: api.Env) -> api.OdooCliFlag:
    """
    Apply the CONVERTER to extract the value of `env` and return all the Odoo args founded
    Args:
        env: The env to convert to OdooCliFlag

    Returns:
        All the args found by the CONVERTER
    """
    store_values = api.OdooCliFlag()
    for converter in CONVERTER:
        with contextlib.suppress(NotImplementedError):
            store_values.update(converter().init(env).to_values())
    return store_values


def env_to_section(section: Type[Section] = None, extra_env: Dict[str, str] = None) -> Section:
    """
    Convert [environnement variables][os.environ] to a
    [Section][odoo_env_config.api.EnvConfigSection], with odoo compatible args,
    by applying a mapper and converter
    Args:
        section: A class inherited from api.EnvConfigSection, declared in CONVERTER
        extra_env: A dict to update environnement variables

    Returns:
        [A Section][odoo_env_config.api.EnvConfigSection] or False if section Class is not found in CONVERTER

    """
    if not section or section not in CONVERTER:
        return False
    curr_env = api.Env(os.environ)
    curr_env.update(extra_env or {})
    curr_env = apply_mapper(env=curr_env)
    return section().init(curr_env)


def env_to_dict(extra_env: Dict[str, str] = None) -> Dict[str, str]:
    """
    Convert [environnement variables][os.environ] to a dict, with odoo compatible args, by applying a mapper and
     converter.
    Args:
        extra_env: A dict to update environnement variables
    Returns:
        A dict with converted environnement variables
    """
    curr_env = api.Env(os.environ)
    curr_env.update(extra_env or {})
    curr_env = apply_mapper(env=curr_env)
    store_values = apply_converter(curr_env)
    return store_values


def env_to_odoo_args(extra_env: Dict[str, str] = None) -> List[str]:
    """
    Entrypoint of this library
    Convert [environnement variable][os.environ] to a odoo args valid.
    See Also
         The env to args [converter][odoo_env_config.api.EnvConfigSection]
         The speccific cloud [env mapper][odoo_env_config.mappers]
    Examples
         >>> import odoo
         >>> odoo.tools.config.parse_args(env_to_odoo_args())
         >>> odoo.tools.config.save()
    Returns:
         A list with args created from Env
    """
    curr_env = api.Env(os.environ)
    curr_env.update(extra_env or {})
    curr_env = apply_mapper(env=curr_env)
    store_values = apply_converter(curr_env)
    return api.dict_to_odoo_args(store_values)


def env_to_config(config: api.OdooConfig, extra_env: Dict[str, str] = None) -> None:
    curr_env = api.Env(os.environ)
    curr_env.update(extra_env or {})
    curr_env = apply_mapper(env=curr_env)
    for converter in CONVERTER:
        with contextlib.suppress(NotImplementedError):
            converter().init(curr_env).write_to_config(config)


def direct_run_command(
    odoo_module: Any,
    force_odoo_args: List[str],
    config_dest: str,
    other_env: api.Env = None,
):
    """
    Entrypoint of the command

    1. First we parse `args`
    2. Then we load `--profiles` if some are provided
    3. And finaly we execute [odoo_env_config][odoo_env_config.entry.env_to_odoo_args] and save it to the dest file

    Args:
        odoo_module: The Odoo module imported
        force_odoo_args: Other args to pass to odoo_module config
        config_dest: The dest file to store the config generated
        other_env: The environment where the config is extracted
    """
    with contextlib.suppress(Exception):
        odoo_module.netsvc.init_logger()

    odoo_module.tools.config.config_file = config_dest
    env_args = env_to_odoo_args(other_env)
    odoo_module.tools.config._parse_config(env_args)
    odoo_module.tools.config._parse_config(force_odoo_args)
    admin_passwd = env_to_section(MiscSection, other_env).admin_password
    if admin_passwd:
        odoo_module.tools.config.set_admin_password(admin_passwd)
    env_to_config(odoo_module.tools.config, other_env)
    odoo_module.tools.config.save()



import unittest

from src.odoo_env_config.api import Env
from src.odoo_env_config.section.limit_section import LimitOdooConfigSection


class TestLimitOdooConfigSection(unittest.TestCase):
    def test_no_value(self):
        conf = LimitOdooConfigSection()
        self.assertEqual(0, conf.limit_request)
        self.assertEqual(0, conf.limit_time_cpu)
        self.assertEqual(0, conf.limit_time_real)
        self.assertEqual(0, conf.osv_memory_count_limit)
        self.assertEqual(0, conf.limit_memory_hard)
        self.assertEqual(0, conf.limit_memory_soft)

    def test_value(self):
        conf = LimitOdooConfigSection().init(
            Env(
                {
                    "LIMIT_REQUEST": str(1),
                    "LIMIT_TIME_CPU": str(2),
                    "LIMIT_TIME_REAL": str(3),
                    "OSV_MEMORY_COUNT_LIMIT": str(4),
                    "LIMIT_MEMORY_HARD": str(5),
                    "LIMIT_MEMORY_SOFT": str(6),
                }
            )
        )
        self.assertEqual(1, conf.limit_request)
        self.assertEqual(2, conf.limit_time_cpu)
        self.assertEqual(3, conf.limit_time_real)
        self.assertEqual(4, conf.osv_memory_count_limit)
        self.assertEqual(5, conf.limit_memory_hard)
        self.assertEqual(6, conf.limit_memory_soft)

    def test_global_hard_default_worker(self):
        conf = LimitOdooConfigSection().init(
            Env(
                {
                    "GLOBAL_LIMIT_MEMORY_HARD": str(1000),
                }
            )
        )
        self.assertEqual(1000, conf.limit_memory_hard)
        self.assertEqual(0, conf.limit_request)
        self.assertEqual(0, conf.limit_time_cpu)
        self.assertEqual(0, conf.limit_time_real)
        self.assertEqual(0, conf.osv_memory_count_limit)
        self.assertEqual(0, conf.limit_memory_soft)

    def test_global_hard_no_worker(self):
        """
        by default 2 worker cron and 1 worker http so we force to 0 worker http and worker cron
        so <GLOBAL_LIMIT_MEMORY_HARD> is divide by 3 (integer way with '//')
        """
        conf = LimitOdooConfigSection().init(
            Env(
                {
                    "WORKER_HTTP": str(0),
                    "WORKER_CRON": str(0),
                    "GLOBAL_LIMIT_MEMORY_HARD": str(1000),
                }
            )
        )
        self.assertEqual(1000, conf.limit_memory_hard)
        self.assertEqual(0, conf.limit_request)
        self.assertEqual(0, conf.limit_time_cpu)
        self.assertEqual(0, conf.limit_time_real)
        self.assertEqual(0, conf.osv_memory_count_limit)
        self.assertEqual(0, conf.limit_memory_soft)

    def test_global_soft_default_worker(self):
        """
        by default 2 worker cron and 1 worker http
        so <GLOBAL_LIMIT_MEMORY_SOFT> is divide by 3 (integer way with '//')
        """
        conf = LimitOdooConfigSection().init(
            Env(
                {
                    "GLOBAL_LIMIT_MEMORY_SOFT": str(1000),
                }
            )
        )
        self.assertEqual(1000, conf.limit_memory_soft)
        self.assertEqual(0, conf.limit_request)
        self.assertEqual(0, conf.limit_time_cpu)
        self.assertEqual(0, conf.limit_time_real)
        self.assertEqual(0, conf.osv_memory_count_limit)
        self.assertEqual(0, conf.limit_memory_hard)

    def test_global_soft_priority(self):
        """
        check priority between <GLOBAL_LIMIT_MEMORY_SOFT> and <LIMIT_MEMORY_SOFT>
        LIMIT_MEMORY_SOFT > GLOBAL_LIMIT_MEMORY_SOFT
        """
        conf = LimitOdooConfigSection().init(
            Env(
                {
                    "LIMIT_MEMORY_SOFT": str(100),
                    "GLOBAL_LIMIT_MEMORY_SOFT": str(1000),
                }
            )
        )
        self.assertEqual(100, conf.limit_memory_soft)
        self.assertEqual(0, conf.limit_request)
        self.assertEqual(0, conf.limit_time_cpu)
        self.assertEqual(0, conf.limit_time_real)
        self.assertEqual(0, conf.osv_memory_count_limit)
        self.assertEqual(0, conf.limit_memory_hard)

    def test_global_hard_priority(self):
        """
        check priority between <GLOBAL_LIMIT_MEMORY_HARD> and <LIMIT_MEMORY_HARD>
        LIMIT_MEMORY_HARD > GLOBAL_LIMIT_MEMORY_HARD
        """
        conf = LimitOdooConfigSection().init(
            Env(
                {
                    "LIMIT_MEMORY_HARD": str(100),
                    "GLOBAL_LIMIT_MEMORY_HARD": str(1000),
                }
            )
        )
        self.assertEqual(100, conf.limit_memory_hard)
        self.assertEqual(0, conf.limit_request)
        self.assertEqual(0, conf.limit_time_cpu)
        self.assertEqual(0, conf.limit_time_real)
        self.assertEqual(0, conf.osv_memory_count_limit)

    def test_global_soft_no_worker(self):
        """
        `GLOBAL_LIMIT_MEMORY_SOFT` is divide by 3 (integer way with '//')
        """
        conf = LimitOdooConfigSection().init(
            Env(
                {
                    "WORKER_HTTP": str(2),
                    "WORKER_CRON": str(1),
                    "GLOBAL_LIMIT_MEMORY_SOFT": str(1000),
                    "LIMIT_MEMORY_HARD": str(200),
                }
            )
        )
        self.assertEqual(333, conf.limit_memory_soft)
        self.assertEqual(0, conf.limit_request)
        self.assertEqual(0, conf.limit_time_cpu)
        self.assertEqual(0, conf.limit_time_real)
        self.assertEqual(0, conf.osv_memory_count_limit)
        self.assertEqual(200, conf.limit_memory_hard)



# Copyright (C) 2023 NDP Systèmes (<http://www.ndp-systemes.fr>).
{
    "name": "Odoo Env to Config",
    "version": "1.0",
    "author": "NDP Systèmes",
    "maintainer": "NDP Systèmes",
    "category": "Module Client",
    "depends": ["base"],
    "description": """
Contient la cli commande pour Odoo pour creer une config depuis l'environment
=============================================================================
""",
    "website": "http://www.ndp-systemes.fr",
    "data": [],
    "demo": [],
    "test": [],
    "installable": False,
    "auto_install": False,
    "license": "AGPL-3",
    "application": False,
}



import doctest
import unittest

from src.odoo_env_config import utils


def suite():
    test_suite = unittest.TestSuite()
    test_suite.addTest(doctest.DocTestSuite(utils))
    return test_suite


if __name__ in ["__main__", "tests.test_doctests"]:
    runner = unittest.TextTestRunner()
    runner.run(suite())



import unittest

from src.odoo_env_config.api import Env
from src.odoo_env_config.section import LoggerSection


class TestLogSection(unittest.TestCase):
    def test_default(self):
        conf = LoggerSection()
        self.assertIsNone(conf.logfile)
        self.assertIsNone(conf.log_handler)
        self.assertFalse(conf.log_request)
        self.assertFalse(conf.log_response)
        self.assertFalse(conf.log_web)
        self.assertFalse(conf.log_sql)
        self.assertFalse(conf.log_db)
        self.assertIsNone(conf.log_db_level)
        self.assertIsNone(conf.log_level)

    def test_global(self):
        conf = LoggerSection().init(
            Env(
                {
                    "LOGFILE": "logfile_value",
                    "LOG_HANDLER": "log_handler_value",
                    "LOG_REQUEST": str(True),
                    "LOG_RESPONSE": str(True),
                    "LOG_WEB": str(True),
                    "LOG_SQL": str(True),
                    "LOG_DB": str(True),
                    "LOG_DB_LEVEL": "log_db_level_value",
                    "LOG_LEVEL": "log_level_value",
                }
            )
        )
        self.assertEqual("logfile_value", conf.logfile)
        self.assertEqual("log_handler_value", conf.log_handler)
        self.assertTrue(conf.log_request)
        self.assertTrue(conf.log_response)
        self.assertTrue(conf.log_web)
        self.assertTrue(conf.log_sql)
        self.assertTrue(conf.log_db)
        self.assertEqual("log_db_level_value", conf.log_db_level)
        self.assertEqual("log_level_value", conf.log_level)

        flags = conf.to_values()
        self.assertIn("logfile", flags)
        self.assertEqual("logfile_value", flags["logfile"])
        self.assertIn("log-handler", flags)
        self.assertEqual("log_handler_value", flags["log-handler"])
        self.assertIn("log-request", flags)
        self.assertTrue(flags["log-request"])
        self.assertIn("log-response", flags)
        self.assertTrue(flags["log-request"])
        self.assertIn("log-web", flags)
        self.assertTrue(flags["log-web"])
        self.assertIn("log-sql", flags)
        self.assertTrue(flags["log-sql"])
        self.assertIn("log-db", flags)
        self.assertTrue(flags["log-db"])
        self.assertIn("log-db-level", flags)
        self.assertEqual("log_db_level_value", flags["log-db-level"])
        self.assertIn("log-level", flags)
        self.assertEqual("log_level_value", flags["log-level"])



import unittest

from src.odoo_env_config.api import Env, OdooCliFlag, ServerUseCase
from src.odoo_env_config.section.worker_section import WorkersOdooConfigSection


class TestWorkersOdooConfigSection(unittest.TestCase):
    def test_default(self):
        conf = WorkersOdooConfigSection()
        self.assertEqual(0, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(0, conf.total)
        conf.init(Env())
        self.assertEqual(0, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(0, conf.total)
        self.assertFalse(conf.to_values())

    def test_WORKER_HTTP(self):
        conf = WorkersOdooConfigSection()
        self.assertEqual(0, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(0, conf.total)
        conf.init(Env({"WORKER_HTTP": str(10)}))
        self.assertEqual(10, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(10, conf.total)
        self.assertEqual(OdooCliFlag({"worker": 10}), conf.to_values())

    def test_WORKER_JOB(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "WORKER_JOB": str(2),
                    "WORKER_CRON": str(1),
                }
            )
        )
        self.assertEqual(0, conf.http)
        self.assertEqual(1, conf.cron)
        self.assertEqual(2, conf.job)
        self.assertEqual(3, conf.total)
        self.assertEqual(
            OdooCliFlag({"worker": 2, "max-cron-thread": 1}), conf.to_values()
        )

    def test_priority(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                }
            )
        )
        self.assertEqual(2, conf.http)
        self.assertEqual(0, conf.cron)  # default value
        self.assertEqual(3, conf.job)
        self.assertEqual(5, conf.total)

    def test_usecase_worker(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "ODOO_USE_CASE": ServerUseCase.WORKER.value,
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                    "WORKER_CRON": str(1),
                }
            )
        )
        self.assertEqual(2, conf.http)
        self.assertEqual(1, conf.cron)
        self.assertEqual(3, conf.job)
        self.assertEqual(6, conf.total)
        self.assertEqual(
            OdooCliFlag({"worker": 5, "max-cron-thread": 1}), conf.to_values()
        )

    def test_usecase_tests(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "ODOO_USE_CASE": ServerUseCase.TESTS.value,
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                    "WORKER_CRON": str(0),
                }
            )
        )
        self.assertEqual(0, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(0, conf.total)
        self.assertEqual(OdooCliFlag(), conf.to_values())
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "TEST_ENABLE": str(True),
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                    "WORKER_CRON": str(0),
                }
            )
        )
        self.assertEqual(0, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(0, conf.total)
        self.assertEqual(OdooCliFlag(), conf.to_values())

    def test_usecase_only_cron(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "ODOO_USE_CASE": ServerUseCase.ONLY_CRON.value,
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                    "WORKER_CRON": str(2),
                }
            )
        )
        self.assertEqual(0, conf.http)
        self.assertEqual(2, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(2, conf.total)
        self.assertEqual(OdooCliFlag({"max-cron-thread": 2}), conf.to_values())

    def test_usecase_only_http(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "ODOO_USE_CASE": ServerUseCase.ONLY_HTTP.value,
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                    "WORKER_CRON": str(1),
                }
            )
        )
        self.assertEqual(2, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(2, conf.total)
        self.assertEqual(OdooCliFlag({"worker": 2}), conf.to_values())

    def test_usecase_only_job_runner(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "ODOO_USE_CASE": ServerUseCase.ONLY_JOB_RUNNER.value,
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                    "WORKER_CRON": str(1),
                }
            )
        )
        self.assertEqual(0, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(0, conf.total)
        self.assertEqual(OdooCliFlag(), conf.to_values())

    def test_usecase_only_job_worker(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "ODOO_USE_CASE": ServerUseCase.ONLY_JOB_WORKER.value,
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                    "WORKER_CRON": str(1),
                }
            )
        )
        self.assertEqual(0, conf.http)
        self.assertEqual(0, conf.cron)
        self.assertEqual(3, conf.job)
        self.assertEqual(3, conf.total)
        self.assertEqual(OdooCliFlag({"worker": 3}), conf.to_values())

    def test_usecase_threaded(self):
        conf = WorkersOdooConfigSection().init(
            Env(
                {
                    "ODOO_USE_CASE": ServerUseCase.THREADED_MODE.value,
                    "WORKER_HTTP": str(2),
                    "WORKER_JOB": str(3),
                    "WORKER_CRON": str(1),
                }
            )
        )
        self.assertEqual(0, conf.http)
        self.assertEqual(1, conf.cron)
        self.assertEqual(0, conf.job)
        self.assertEqual(1, conf.total)
        self.assertEqual(OdooCliFlag({"max-cron-thread": 1}), conf.to_values())



# -*- coding: utf8 -*-
import unittest

from src.odoo_env_config import api
from src.odoo_env_config.mappers import compatibility


class TestOdooCompatibilityMapper(unittest.TestCase):
    def test_empty(self):
        result = compatibility(api.Env())
        self.assertEqual(9, len(result.keys()))
        for key_result, value in result.items():
            self.assertIsNone(value, "Value of key [%s] is not None" % key_result)

    def test_not_change(self):
        to_map = {
            "WORKER_HTTP": "2",
            "WORKERS": "3",
        }
        result = compatibility(api.Env(to_map))
        self.assertEqual(10, len(result.keys()))
        self.assertEqual("2", result["WORKER_HTTP"])
        self.assertIsNone(result["WORKER_JOB"])
        self.assertIsNone(result["WORKER_CRON"])
        self.assertIsNone(result["HTTP_INTERFACE"])
        self.assertIsNone(result["HTTP_PORT"])
        self.assertIsNone(result["HTTP_ENABLE"])
        self.assertIsNone(result["LONGPOLLING_PORT"])
        self.assertIsNone(result["SERVER_WIDE_MODULES"])

    def test_workers_in_worker_http(self):
        value = "2"
        to_map = {
            "WORKER_HTTP": value,
            # "WORKERS": value,
        }
        result = compatibility(api.Env(to_map))
        self.assertEqual(9, len(result.keys()))
        self.assertEqual(value, result["WORKER_HTTP"])
        self.assertIsNone(result["WORKER_JOB"])
        self.assertIsNone(result["WORKER_CRON"])
        self.assertIsNone(result["HTTP_INTERFACE"])
        self.assertIsNone(result["HTTP_PORT"])
        self.assertIsNone(result["HTTP_ENABLE"])
        self.assertIsNone(result["LONGPOLLING_PORT"])
        self.assertIsNone(result["SERVER_WIDE_MODULES"])

    def test_worker_http_in_workers(self):
        value = "3"
        to_map = {
            # "WORKER_HTTP": value,
            "WORKERS": value,
        }
        result = compatibility(api.Env(to_map))
        self.assertEqual(10, len(result.keys()))
        self.assertEqual(value, result["WORKER_HTTP"])
        self.assertIsNone(result["WORKER_JOB"])
        self.assertIsNone(result["WORKER_CRON"])
        self.assertIsNone(result["HTTP_INTERFACE"])
        self.assertIsNone(result["HTTP_PORT"])
        self.assertIsNone(result["HTTP_ENABLE"])
        self.assertIsNone(result["LONGPOLLING_PORT"])
        self.assertIsNone(result["SERVER_WIDE_MODULES"])

    def test_gevent_port(self):
        result = compatibility(api.Env({
            "GEVENT_PORT": "7894",
            "LONGPOLLING_PORT": "1234",
        }))
        self.assertEqual(result["LONGPOLLING_PORT"], "7894")
        result = compatibility(api.Env({
            "LONGPOLLING_PORT": "1234",
        }))
        self.assertEqual(result["LONGPOLLING_PORT"], "1234")
        result = compatibility(api.Env({
            "GEVENT_PORT": "1234",
        }))
        self.assertEqual(result["LONGPOLLING_PORT"], "1234")

    def test_server_wide_modules(self):
        value = "module"
        to_map = {
            # "WORKER_HTTP": value,
            "SERVER_WIDE_MODULES": value,
        }
        result = compatibility(api.Env(to_map))
        self.assertEqual(9, len(result.keys()))
        self.assertEqual(value, result["SERVER_WIDE_MODULES"])
        self.assertIsNone(result["WORKER_HTTP"])
        self.assertIsNone(result["WORKER_JOB"])
        self.assertIsNone(result["WORKER_CRON"])
        self.assertIsNone(result["HTTP_INTERFACE"])
        self.assertIsNone(result["HTTP_PORT"])
        self.assertIsNone(result["HTTP_ENABLE"])
        self.assertIsNone(result["LONGPOLLING_PORT"])

        value = "module"
        to_map = {
            # "WORKER_HTTP": value,
            "LOAD": value,
        }
        result = compatibility(api.Env(to_map))
        self.assertEqual(10, len(result.keys()))
        self.assertEqual(value, result["SERVER_WIDE_MODULES"])
        self.assertIsNone(result["WORKER_HTTP"])
        self.assertIsNone(result["WORKER_JOB"])
        self.assertIsNone(result["WORKER_CRON"])
        self.assertIsNone(result["HTTP_INTERFACE"])
        self.assertIsNone(result["HTTP_PORT"])
        self.assertIsNone(result["HTTP_ENABLE"])
        self.assertIsNone(result["LONGPOLLING_PORT"])

    def test_http_config_xmlrpc(self):
        HTTP_INTERFACE = "127.0.0.1"
        HTTP_PORT = "8080"
        HTTP_ENABLE = "True"
        LONGPOLLING_PORT = "4040"
        to_map = {
            "XMLRPC_INTERFACE": HTTP_INTERFACE,
            "XMLRPC_PORT": HTTP_PORT,
            "XMLRPC_ENABLE": HTTP_ENABLE,
            "LONGPOLLING_PORT": LONGPOLLING_PORT,
        }

        result = compatibility(api.Env(to_map))
        self.assertIsNone(result["WORKER_HTTP"])
        self.assertIsNone(result["WORKER_JOB"])
        self.assertIsNone(result["WORKER_CRON"])
        self.assertEqual(HTTP_INTERFACE, result["HTTP_INTERFACE"])
        self.assertEqual(HTTP_PORT, result["HTTP_PORT"])
        self.assertEqual(HTTP_ENABLE, result["HTTP_ENABLE"])
        self.assertEqual(LONGPOLLING_PORT, result["LONGPOLLING_PORT"])
        self.assertIsNone(result["SERVER_WIDE_MODULES"])

    def test_http_config_http(self):
        HTTP_INTERFACE = "127.0.0.1"
        HTTP_PORT = "8080"
        HTTP_ENABLE = "True"
        LONGPOLLING_PORT = "4040"
        to_map = {
            "HTTP_INTERFACE": HTTP_INTERFACE,
            "HTTP_PORT": HTTP_PORT,
            "HTTP_ENABLE": HTTP_ENABLE,
            "LONGPOLLING_PORT": LONGPOLLING_PORT,
        }
        result = compatibility(api.Env(to_map))
        self.assertEqual(HTTP_INTERFACE, result["HTTP_INTERFACE"])
        self.assertEqual(HTTP_PORT, result["HTTP_PORT"])
        self.assertEqual(HTTP_ENABLE, result["HTTP_ENABLE"])
        self.assertEqual(LONGPOLLING_PORT, result["LONGPOLLING_PORT"])
        self.assertIsNone(result["SERVER_WIDE_MODULES"])

    def test_priority(self):
        HTTP_INTERFACE = "127.0.0.1"
        HTTP_PORT = "8080"
        HTTP_ENABLE = "True"
        to_map = {
            "WORKER_HTTP": "1",
            "WORKER_CRON": "2",
            "WORKER_JOB": "3",
            "WORKERS": "4",
            "HTTP_INTERFACE": HTTP_INTERFACE,
            "XMLRPC_INTERFACE": HTTP_INTERFACE + "_fake",
            "HTTP_PORT": HTTP_PORT,
            "XMLRPC_PORT": HTTP_PORT + "_fake",
            "HTTP_ENABLE": HTTP_ENABLE,
            "XMLRPC_ENABLE": HTTP_ENABLE + "_fake",
            "SERVER_WIDE_MODULES": "module",
            "LOAD": "module_fake",
        }
        result = compatibility(api.Env(to_map))
        self.assertEqual(HTTP_INTERFACE, result["HTTP_INTERFACE"])
        self.assertEqual(HTTP_PORT, result["HTTP_PORT"])
        self.assertEqual(HTTP_ENABLE, result["HTTP_ENABLE"])
        self.assertEqual("module", result["SERVER_WIDE_MODULES"])
        self.assertIsNone(result["LONGPOLLING_PORT"])
        self.assertEqual("1", result["WORKER_HTTP"])
        self.assertEqual("2", result["WORKER_CRON"])
        self.assertEqual(
            "3", result["WORKER_JOB"], "WORKER_JOB prioritaire sur WORKERS"
        )



import logging
import pathlib
from typing import List

from addons_installer import addons_installer
from typing_extensions import Self

from .. import api

_logger = logging.getLogger(__name__)


class AddonsPathConfigSection(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.addons_path: List[str] = []

    def init(self, curr_env: api.Env) -> Self:
        results = addons_installer.AddonsFinder.parse_env(env_vars=curr_env)
        self.addons_path = []
        for result in results:
            path = pathlib.Path(result.addons_path)
            subs = addons_installer.AddonsFinder.parse_submodule([result])
            self.addons_path.extend(map(lambda it: it.addons_path, subs))
            if (path / "EXCLUDE").exists():
                # EXCLUDE not exclude submodule discover
                # Only exclude this module from the addon-path
                _logger.info("Ignore %s with EXCLUDE file", result.addons_path)
                continue
            self.addons_path.append(result.addons_path)
        return self

    def to_values(self) -> api.OdooCliFlag:
        res = api.OdooCliFlag()
        res.set("addons-path", self.addons_path)
        return res



"""
Contains de converter from [os.environ][os.environ] wrapper in
[][odoo_env_config.api.Env] in this library to valid `odoo.tools.config` options
"""

from __future__ import annotations

import enum
from typing import Union

from typing_extensions import Self

from .. import api
from .worker_section import WorkersOdooConfigSection as _WorkersOdooConfigSection


class MaxConnMode(enum.Enum):
    """
    Mode to compute the max_conn attribute
    Attributes:
        AUTO: the max_conn is compute from the number of worker defined
        in [`WorkerConfig`][odoo_env_config.section.worker_section]
        FIXED: the value is taken from `"DB_MAX_CONN"` is [][os.environ]
    """

    AUTO = "AUTO"
    FIXED = "FIXED"


def compute_auto_maxconn(curr_env: api.Env) -> int:
    """
    Compute the current maxconn based on the number of worker
    Odoo recomendation is ~= Number of worker * 1.5.
    Args:
        curr_env: The current Env

    Returns:
        The number of worker * 1.5
    """
    nb_workers = _WorkersOdooConfigSection().init(curr_env).worker
    return nb_workers + int(nb_workers // 2)


class DatabaseOdooConfigSection(api.EnvConfigSection):
    """
    A dataclass to hold the database Odoo config group option

    Attributes:
        name: Name of the database
        host: The URI of the postgresql service get from `"DB_HOST"` in
        user: The URI of the postgresql service get from `"DB_HOST"` in
        port: The URI of the postgresql service get from `"DB_HOST"` in
        password: The URI of the postgresql service get from `"DB_HOST"` in
        max_conn: The URI of the postgresql service get from `"DB_HOST"` in
        filter: The URI of the postgresql service get from `"DB_HOST"` in
        log_enable: The URI of the postgresql service get from `"DB_HOST"` in
        log_level: The URI of the postgresql service get from `"DB_HOST"` in
        show: The URI of the postgresql service get from `"DB_HOST"` in
        maxconn_mode: The URI of the postgresql service get from `"DB_HOST"` in
    """

    def __init__(self):
        super().__init__()
        self.name: Union[str, None] = None
        self.host: Union[str, None] = None
        self.port: int = 0
        self.user: Union[str, None] = None
        self.password: Union[str, None] = None
        self.max_conn: int = 0
        self.filter: Union[str, None] = None
        self.log_level: Union[str, None] = None
        self.log_enable: bool = False
        self.show: bool = True
        self.maxconn_mode: MaxConnMode = MaxConnMode.AUTO

    def init(self, curr_env: api.Env) -> Self:
        """
        Args:
            curr_env: The current env to parse
        Returns:
            A new `DatabaseOdooConfigSection`
        """

        self.name = curr_env.get("DB_NAME")
        self.host = curr_env.get("DB_HOST")
        self.port = curr_env.get_int("DB_PORT")
        self.user = curr_env.get("DB_USER")
        self.password = curr_env.get("DB_PASSWORD")
        self.max_conn = curr_env.get_int("DB_MAX_CONN")
        self.filter = curr_env.get("DB_FILTER")
        self.log_enable = curr_env.get_bool("LOG_DB")
        self.log_level = curr_env.get("LOG_DB_LEVEL")
        self.show = curr_env.get_bool("LIST_DB", default=True)
        # Determine max_conn
        self.maxconn_mode = curr_env.get_enum("DB_MAX_CONN_MODE", MaxConnMode, default=MaxConnMode.FIXED)
        if self.maxconn_mode == MaxConnMode.AUTO:
            self.max_conn = max(self.max_conn, compute_auto_maxconn(curr_env))

        if not self.filter and not self.show:
            self.filter = self.name
        return self

    def to_values(self) -> api.OdooCliFlag:
        res = api.OdooCliFlag()
        res.set_all(
            {
                "db_host": self.host,
                "db_port": self.port,
                "db_user": self.user,
                "db_password": self.password,
                "database": self.name,
                "no-database-list": not self.show,
                "db_maxconn": self.max_conn,
                "db-filter": self.filter,
            }
        )
        return res



import unittest

from src.odoo_env_config.api import Env
from src.odoo_env_config.section import UpdateInstallSection
from tests._decorators import MultiOdooVersion


class TestUpdateInitSection(unittest.TestCase):
    @MultiOdooVersion.without_args
    def test_default(self):
        conf = UpdateInstallSection()
        self.assertEqual([], conf.install)
        self.assertEqual([], conf.update)
        flags = conf.to_values()
        self.assertFalse(flags)

    @MultiOdooVersion.with_args
    def test_value(self, version: int):
        conf = UpdateInstallSection().init(
            Env(
                {
                    "ODOO_VERSION": str(version),
                    "INSTALL": " module1 , module2, module3 , module1 ",
                    "UPDATE": " module_a , module_b , module_c, module_b ",
                }
            )
        )
        flags = conf.to_values()

        self.assertListEqual(["module1", "module2", "module3"], conf.install)
        self.assertIn("init", flags)
        self.assertEqual("module1,module2,module3", flags["init"])

        self.assertListEqual(["module_a", "module_b", "module_c"], conf.update)
        self.assertIn("update", flags)
        self.assertEqual("module_a,module_b,module_c", flags["update"])



import unittest

from src.odoo_env_config.api import Env, OdooCliFlag
from src.odoo_env_config.section import ServerWideModuleConfigSection


class TestHttpOdooConfigSection(unittest.TestCase):
    def test_default(self):
        conf = ServerWideModuleConfigSection()
        self.assertEqual(OdooCliFlag(), conf.to_values())
        self.assertListEqual([], conf.server_wide_modules)

    def test_serer_wide_modules_key(self):
        conf = ServerWideModuleConfigSection().init(
            Env({"SERVER_WIDE_MODULES": "module1,module2"})
        )
        self.assertEqual(["module1", "module2"], conf.server_wide_modules)
        conf = ServerWideModuleConfigSection().init(
            Env({"SERVER_WIDE_MODULES": "module1,module2,module1,base"})
        )
        # Assert no duplicate module and order is keeped
        self.assertEqual(["module1", "module2", "base"], conf.server_wide_modules)

    def test_module_name(self):
        conf = ServerWideModuleConfigSection().init(
            Env(
                {
                    "LOAD_MODULE_A": str(True),
                    "LOAD_MODULE_0": str(1),
                    "LOAD_MODULE_1": str(False),
                    "LOAD_MODULE_2": str(0),
                }
            )
        )
        # Assert "True" or "1" is valid activate value, and the sort is alpha
        self.assertEqual(["module_a", "module_0"], conf.server_wide_modules)
        conf = ServerWideModuleConfigSection().init(
            Env(
                {
                    "LOAD_QUEUE_JOB": "my_custom_module",
                }
            )
        )
        self.assertEqual(["my_custom_module"], conf.server_wide_modules)
        conf = ServerWideModuleConfigSection().init(
            Env(
                {
                    "LOAD_aslkaskalds": "queue_job",
                }
            )
        )
        self.assertEqual(["queue_job"], conf.server_wide_modules)



from typing_extensions import Self

from .. import api
from ..api import OdooVersion


class TestOdooConfigSection(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.enable = False
        self.test_tags = None
        self.test_file = None

    def init(self, curr_env: api.Env) -> Self:
        self.enable = curr_env.get_bool("TEST_ENABLE")
        self.test_tags = curr_env.get("TEST_TAGS")
        if curr_env.odoo_version <= OdooVersion.V11.value:
            self.test_tags = None  # Not supported in Odoo 11 or less
        self.test_file = curr_env.get("TEST_FILE")
        return self

    def to_values(self) -> api.OdooCliFlag:
        res = api.OdooCliFlag()
        res.set("test-enable", self.enable)
        if not self.enable:
            return res
        res.set("test-tags", self.test_tags)
        res.set("test-file", self.test_file)
        return res



import configparser
import os
import unittest
import uuid
from os.path import join as pjoin
from typing import Any, Dict, List, Tuple

from odoo_env_config import api, cli


def _try_find_addons_info() -> Tuple[str, int]:
    """
    Try to find the odoo path to be able to run the test inside Odoo on local host or inside a odoo docker image

    In local machine :
    1. You need to run the test in the correct odoo virtualenv
    2. Install this library with `pip install -e .` in the correct odoo virtualenv
    3. Add ODOO_VERSION equal the version you are in

    Returns:
        The absolute path of odoo-bin, the odoo version
    Raises:
        FileNotFoundError if no odoo-bin is found

    """
    current_env = api.Env(os.environ)
    odoo_version: int = current_env.odoo_version
    odoo_path = current_env.odoo_bin
    if current_env.odoo_version:
        # If the odoo-bin file not exist, maybe we are in a dev computer.
        # So we try to find where the odoo-bin is installed
        if not os.path.exists(current_env.odoo_bin):
            _base_path = "~/workspace/python/odoo/"
            odoov_path = pjoin(
                os.path.expanduser(_base_path),
                f"v{odoo_version}",
                f"odoo{odoo_version}",
                "odoo-bin",
            )
            if os.path.exists(odoov_path):
                odoo_path = odoov_path
    if not os.path.exists(odoo_path):
        raise FileNotFoundError(odoo_path)
    return odoo_path, odoo_version


def create_config(
    env_files: List[str] = None, odoo_rc: str = None, extra_odoo_args: List[str] = None
) -> configparser.ConfigParser:
    """
    This function create a config file using the subprocess launching odoo of the version

    Args:
        env_files: The profile you want to add
        odoo_rc: To force where the odoo-rc is generated
        extra_odoo_args: Extra Odoo Args you want to add instead of Env var
    Returns:
        The odoo-rc generated if success
    Raises:
        ValueError: if the subprocess calling oenv2config failed
    """
    odoo_bin_path, odoo_version = _try_find_addons_info()
    rand = str(uuid.uuid4()).split("-")[0]
    odoo_rc = odoo_rc or f"/tmp/odoo_rc-{rand}.ini"
    print("rc file in ", odoo_rc)
    if os.path.exists(odoo_rc):
        os.remove(odoo_rc)  # Remove to sure no file exist
    profiles = []
    for env_file in env_files or []:
        efile = pjoin(os.path.dirname(__file__), "profiles", f"{env_file}.env")
        if os.path.exists(efile):
            profiles.append(efile)
        efile_v = pjoin(
            os.path.dirname(__file__), "profiles", f"{env_file}-{odoo_version}.env"
        )
        if os.path.exists(efile_v):
            profiles.append(efile_v)

    res_code = cli.execute_odoo_cli_helper(
        odoo_bin_path, profiles, odoo_rc, extra_odoo_args or []
    )
    if res_code:
        raise ValueError("Error on launch odoo env to config")
    parser = configparser.ConfigParser()
    parser.read(odoo_rc)
    return parser


def assertParser(
    case: unittest.TestCase,
    parser: configparser.ConfigParser,
    expected_values: Dict[str, Any],
    section="options",
):
    """
    Assert the parser comparing the expectedValue.
    Do some typing converstion based on the type of the value of the key in expected_values
    If the value in `expected_values` to compare to the same key in parser is:
    - bool, the `parser.getboolean` is used with `assertEqual`
    - int, the `parser.getint` is used with `assertEqual`
    - float, the function `parser.getfloat` is used with `assertEqual`
    - List, a split on `,` if applied and `assertListEqual`
    - other `assertEqual` without conversion

    Args:
        case: The test case
        parser: The parser to assert
        expected_values: The key, value to assert
        section: If you want to assert an another section
    """
    case.assertTrue(parser)
    case.assertTrue(expected_values)
    dict_compare = {}
    for key, value in expected_values.items():
        pvalue = parser.get(section, key)
        if isinstance(value, bool):
            try:
                pvalue = parser.getboolean(section, key)
            except ValueError:
                case.fail(
                    f"{section}.{key}, not an 'boolean' value, {parser.get(section, key)}"
                )
        elif isinstance(value, int):
            try:
                pvalue = parser.getint(section, key)
            except ValueError:
                case.fail(
                    f"{section}.{key}, not an 'int' value, '{parser.get(section, key)}'"
                )
        elif isinstance(value, float):
            try:
                pvalue = parser.getfloat(section, key)
            except ValueError:
                case.fail(
                    f"{section}.{key}, not an 'float' value, {parser.get(section, key)}"
                )

        elif isinstance(value, list):
            pvalue = parser.get(section, key).split(",")
        case.assertEqual(pvalue, value, f"{section}.{key}, not equal")
        dict_compare[key] = pvalue
    case.assertDictEqual(expected_values, dict_compare)



import unittest

from src.odoo_env_config.api import Env
from src.odoo_env_config.section import AddonsPathConfigSection
from tests._decorators import MultiOdooVersion


class TestDatabaseOdooConfigSection(unittest.TestCase):
    @MultiOdooVersion.without_args
    def test_default(self):
        conf = AddonsPathConfigSection()
        self.assertListEqual([], conf.addons_path)
        flags = conf.to_values()
        self.assertFalse(flags)

    @MultiOdooVersion.with_args
    def test_global(self, version):
        conf = AddonsPathConfigSection().init(
            Env(
                {
                    "ODOO_VERSION": str(version),
                    "ADDONS_GIT_DEFAULT_SERVER": "github.com",
                    "ADDONS_GIT_MODULE_1": "my_module",
                    "ADDONS_LOCAL_RELATIVE_MODULE_2": "path/to/module",
                    "ADDONS_LOCAL_RELATIVE_MODULE_2_BASE_PATH": "/base/relative",
                    "ADDONS_LOCAL_ABS_PATH_MODULE_3": "/path/to/module3",
                }
            )
        )
        flags = conf.to_values()
        self.assertIn("addons-path", flags)
        self.assertEqual(3, len(conf.addons_path))
        self.assertIn("/odoo/addons/my_module", flags["addons-path"])
        self.assertIn("/base/relative/path/to/module", flags["addons-path"])
        self.assertIn("/path/to/module3", flags["addons-path"])



class OdooLauncherException(Exception):
    pass


class MissingDBCredential(OdooLauncherException):
    def __init__(self) -> None:
        super(MissingDBCredential, self).__init__()
        self.msg = """Can't start Odoo without a db name
        Please add the one of the following environment variable
        - DATABASE
        - DB_NAME
        - POSTGRESQL_ADDON_DB""".lstrip()



import os
import unittest

from src.odoo_env_config import api
from src.odoo_env_config.mappers import clevercloud_cellar
from tests._decorators import MultiOdooVersion


class TestCleverCloudMapperCellar(unittest.TestCase):
    def _test_priority(self, key, to_map, expected):
        result = clevercloud_cellar(api.Env(to_map))
        self.assertIn(key, result)
        self.assertEqual(
            expected,
            result[key],
            "Value of key [%s] is not expected '%s' result : '%s'"
            % (key, expected, result[key]),
        )

    @MultiOdooVersion.with_args
    def test_map_cellar(self, version):
        host = "my-host.com"
        secret_key = "SECRET_KEY"
        key_id = "KEY_ID"
        region = "fr-par"
        result = clevercloud_cellar(
            api.Env(
                {
                    "ODOO_VERSION": str(version),
                    "CELLAR_ADDON_HOST": host,
                    "CELLAR_ADDON_KEY_SECRET": secret_key,
                    "CELLAR_ADDON_KEY_ID": key_id,
                    "CELLAR_ADDON_REGION": region,
                }
            )
        )
        self.assertEqual(11, len(result.keys()))

    @MultiOdooVersion.with_args
    def test_map_minio(self, version):
        host = "my-host.com"
        secret_key = "SECRET_KEY"
        key_id = "KEY_ID"
        secure = False
        result = clevercloud_cellar(
            api.Env(
                {
                    "ODOO_VERSION": str(version),
                    "MINIO_DOMAIN": host,
                    "MINIO_SECRET_KEY": secret_key,
                    "MINIO_ACCESS_KEY": key_id,
                    "S3_FILESTORE_SECURE": secure,
                }
            )
        )
        self.assertEqual(11, len(result.keys()))

    def test_service_minio(self):
        env = api.Env(os.environ)
        if "MINIO_ACCESS_KEY" in env.keys():
            result = clevercloud_cellar(env)
            self.assertTrue(result["S3_FILESTORE_HOST"])
            self.assertTrue(result["S3_FILESTORE_SECRET_KEY"])
            self.assertTrue(result["S3_FILESTORE_ACCESS_KEY"])
            self.assertTrue(result["S3_SECURE"])

    @MultiOdooVersion.with_args
    def test_host(self, version):
        """
        Host definition argument are mapped to S3_... corresponding parameters
        """
        host = "my-host.com"
        secret_key = "SECRET_KEY"
        key_id = "KEY_ID"
        region = "fr-par"
        result = clevercloud_cellar(
            api.Env(
                {
                    "ODOO_VERSION": str(version),
                    "CELLAR_ADDON_HOST": host,
                    "CELLAR_ADDON_KEY_SECRET": secret_key,
                    "CELLAR_ADDON_KEY_ID": key_id,
                    "CELLAR_ADDON_REGION": region,
                }
            )
        )
        self.assertEqual(
            11,
            len(result.keys()),
            "mapper has added 4 parameters S3_FILESTORE_HOST, S3_...",
        )
        self.assertEqual(host, result["S3_FILESTORE_HOST"])
        self.assertEqual(secret_key, result["S3_FILESTORE_SECRET_KEY"])
        self.assertEqual(key_id, result["S3_FILESTORE_ACCESS_KEY"])
        self.assertEqual(region, result["S3_FILESTORE_REGION"])

    @MultiOdooVersion.with_args
    def test_priority(self, version):
        """
        Test que les variables mapper sont prioritaires si elles existent
        """

        host = "my-host.com"
        secret_key = "SECRET_KEY"
        key_id = "KEY_ID"
        region = "fr-par"
        origin = "_origin"
        result = clevercloud_cellar(
            api.Env(
                {
                    "ODOO_VERSION": str(version),
                    "CELLAR_ADDON_HOST": host,
                    "S3_FILESTORE_HOST": host + origin,
                    "S3_FILESTORE_SECRET_KEY": secret_key + origin,
                    "CELLAR_ADDON_KEY_SECRET": secret_key,
                    "S3_FILESTORE_ACCESS_KEY": key_id + origin,
                    "CELLAR_ADDON_KEY_ID": key_id,
                    "S3_FILESTORE_REGION": region + origin,
                    "CELLAR_ADDON_REGION": region,
                }
            )
        )
        self.assertEqual(11, len(result.keys()))
        self.assertEqual(host + origin, result["S3_FILESTORE_HOST"])
        self.assertEqual(secret_key + origin, result["S3_FILESTORE_SECRET_KEY"])
        self.assertEqual(key_id + origin, result["S3_FILESTORE_ACCESS_KEY"])
        self.assertEqual(region + origin, result["S3_FILESTORE_REGION"])

    @MultiOdooVersion.with_args
    def test_default_region(self, version):
        host = "my-host.com"
        secret_key = "SECRET_KEY"
        key_id = "KEY_ID"
        result = clevercloud_cellar(
            api.Env(
                {
                    "ODOO_VERSION": str(version),
                    "CELLAR_ADDON_HOST": host,
                    "CELLAR_ADDON_KEY_SECRET": secret_key,
                    "CELLAR_ADDON_KEY_ID": key_id,
                }
            )
        )
        self.assertEqual(
            10,
            len(result.keys()),
            "mapper has set S3_FILESTORE_ENABLE and generated 4 S3_xx parameter",
        )
        self.assertEqual(host, result["S3_FILESTORE_HOST"])
        self.assertEqual(secret_key, result["S3_FILESTORE_SECRET_KEY"])
        self.assertEqual(key_id, result["S3_FILESTORE_ACCESS_KEY"])
        self.assertIsNone(
            result["S3_FILESTORE_REGION"],
            "internal default value has been used by mapper",
        )

    @MultiOdooVersion.with_args
    def test_priority_s3_host(self, version):
        """
        Test the priority of the key
        S3_FILESTORE_HOST > CELLAR_ADDON_HOST
        """
        value1 = "value1"
        value2 = "value2"
        value3 = "value3"
        self._test_priority(
            key="S3_FILESTORE_HOST",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_HOST": value3,
                "S3_FILESTORE_HOST": value1,
                "CELLAR_ADDON_HOST": value2,
            },
            expected=value1,
        )
        to_map2 = {
            "ODOO_VERSION": str(version),
            "ODOO_S3_HOST": value3,
            "CELLAR_ADDON_HOST": value2,
        }
        self._test_priority(
            key="S3_FILESTORE_HOST",
            to_map=to_map2,
            expected=value2,
        )
        self._test_priority(
            key="S3_FILESTORE_HOST",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_HOST": value3,
            },
            expected=value3,
        )

    @MultiOdooVersion.with_args
    def test_priority_s3_secret_key(self, version):
        """
        Test the priority of the key
        S3_FILESTORE_SECRET_KEY > CELLAR_ADDON_KEY_SECRET
        """
        value1 = "value1"
        value2 = "value2"
        value3 = "value3"
        self._test_priority(
            key="S3_FILESTORE_SECRET_KEY",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_SECRET_KEY": value3,
                "S3_FILESTORE_SECRET_KEY": value1,
                "CELLAR_ADDON_KEY_SECRET": value2,
            },
            expected=value1,
        )
        self._test_priority(
            key="S3_FILESTORE_SECRET_KEY",
            to_map={
                "ODOO_VERSION": str(version),
                "CELLAR_ADDON_KEY_SECRET": value2,
                "ODOO_S3_SECRET_KEY": value3,
            },
            expected=value2,
        )
        self._test_priority(
            key="S3_FILESTORE_SECRET_KEY",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_SECRET_KEY": value3,
            },
            expected=value3,
        )

    @MultiOdooVersion.with_args
    def test_priority_s3_acces_key(self, version):
        """
        Test the priority of the key
        S3_FILESTORE_ACCESS_KEY > CELLAR_ADDON_KEY_ID
        """
        value1 = "value1"
        value2 = "value2"
        value3 = "value3"
        self._test_priority(
            key="S3_FILESTORE_ACCESS_KEY",
            to_map={
                "ODOO_VERSION": str(version),
                "CELLAR_ADDON_KEY_ID": value2,
                "ODOO_S3_ACCESS_KEY": value3,
                "S3_FILESTORE_ACCESS_KEY": value1,
            },
            expected=value1,
        )
        self._test_priority(
            key="S3_FILESTORE_ACCESS_KEY",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_ACCESS_KEY": value3,
                "CELLAR_ADDON_KEY_ID": value2,
            },
            expected=value2,
        )
        self._test_priority(
            key="S3_FILESTORE_ACCESS_KEY",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_ACCESS_KEY": value3,
            },
            expected=value3,
        )

    @MultiOdooVersion.with_args
    def test_priority_s3_region(self, version):
        """
        Test the priority of the key
        S3_FILESTORE_REGION > CELLAR_ADDON_REGION
        """
        value1 = "value1"
        value2 = "value2"
        value3 = "value3"
        self._test_priority(
            key="S3_FILESTORE_REGION",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_REGION": value3,
                "S3_FILESTORE_REGION": value1,
                "CELLAR_ADDON_REGION": value2,
            },
            expected=value1,
        )
        self._test_priority(
            key="S3_FILESTORE_REGION",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_REGION": value3,
                "CELLAR_ADDON_REGION": value2,
            },
            expected=value2,
        )
        self._test_priority(
            key="S3_FILESTORE_REGION",
            to_map={
                "ODOO_VERSION": str(version),
                "ODOO_S3_REGION": value3,
            },
            expected=value3,
        )



from typing import Union

from typing_extensions import Self

from .. import api
from ..api import OdooVersion


class HttpOdooConfigSection(api.EnvConfigSection):
    """
    This section contains the mapping of `HTTP Service Configuration` from the odoo config parser
    """

    def __init__(self):
        super().__init__()
        self.enable: bool = False
        self.proxy_mode: bool = False
        self.interface: Union[str, None] = None
        self.port: int = 0
        self.longpolling_port: int = 0
        self.enable_x_sendfile: bool = False
        self._gevent_config_name = "longpolling-port"

    def init(self, curr_env: api.Env) -> Self:
        self.enable = curr_env.get_bool("HTTP_ENABLE", default=True)
        self.interface = curr_env.get("HTTP_INTERFACE")
        self.port = curr_env.get_int("HTTP_PORT")
        self.proxy_mode = curr_env.get_bool("PROXY_ENABLE", "PROXY_MODE")
        self.longpolling_port = curr_env.get_int("LONGPOLLING_PORT")
        if curr_env.odoo_version_type >= OdooVersion.V16:
            self._gevent_config_name = "gevent-port"
        if curr_env.odoo_version_type >= OdooVersion.V16:
            self.enable_x_sendfile = curr_env.get_bool("X_SENDFILE")
        return self

    def to_values(self) -> api.OdooCliFlag:
        res = api.OdooCliFlag()
        if not self.enable:
            return res.set("no-http", True)

        res.set("http-interface", self.interface)
        res.set("http-port", self.port)
        res.set(self._gevent_config_name or "", self.longpolling_port)
        res.set("x-sendfile", self.enable_x_sendfile)
        res.set("proxy-mode", self.proxy_mode)
        return res



# -*- coding: utf8 -*-
import unittest

from src.odoo_env_config import api, entry
from src.odoo_env_config.section.db_section import DatabaseOdooConfigSection


class TestOdooConfig(unittest.TestCase):
    def test_no_config(self):
        config = entry.env_to_odoo_args()
        self.assertFalse(config)

    def test_args_database(self):
        db_host = "my-host.com"
        db_name = "my-db"
        db_port = str(5253)
        db_user = "my-user"
        db_password = "py-password"
        extra_env = {
            "POSTGRESQL_ADDON_DB": db_name,
            "POSTGRESQL_ADDON_HOST": db_host,
            "POSTGRESQL_ADDON_PORT": db_port,
            "POSTGRESQL_ADDON_USER": db_user,
            "POSTGRESQL_ADDON_PASSWORD": db_password,
        }
        args = entry.env_to_odoo_args(extra_env)
        self.assertSetEqual(
            {
                "--db_host=" + db_host,
                "--db_port=" + db_port,
                "--db_user=" + db_user,
                "--db_password=" + db_password,
                "--database=" + db_name,
            },
            set(args),
        )

    def test_args_direct_database(self):
        """
        Assert Mapper for direct acces postgres provide by clevercloud
        """
        db_host = "my-host.com"
        db_name = "my-db"
        db_port = str(5253)
        db_user = "my-user"
        db_password = "py-password"
        extra_env = {
            "POSTGRESQL_ADDON_DB": db_name,
            "POSTGRESQL_ADDON_DIRECT_HOST": db_host,
            "POSTGRESQL_ADDON_DIRECT_PORT": db_port,
            "POSTGRESQL_ADDON_USER": db_user,
            "POSTGRESQL_ADDON_PASSWORD": db_password,
        }
        args = entry.env_to_odoo_args(extra_env)
        self.assertSetEqual(
            {
                "--db_host=" + db_host,
                "--db_port=" + db_port,
                "--db_user=" + db_user,
                "--db_password=" + db_password,
                "--database=" + db_name,
            },
            set(args),
        )

    def test_env_to_dict_no_config(self):
        config = entry.env_to_dict()
        self.assertFalse(config)

    def test_env_to_dict_db_config(self):
        db_host = "my-host.com"
        db_name = "my-db"
        db_port = 5253
        db_user = "my-user"
        db_password = "py-password"
        extra_env = {
            "POSTGRESQL_ADDON_DB": db_name,
            "POSTGRESQL_ADDON_HOST": db_host,
            "POSTGRESQL_ADDON_PORT": db_port,
            "POSTGRESQL_ADDON_USER": db_user,
            "POSTGRESQL_ADDON_PASSWORD": db_password,
        }
        dict_args = entry.env_to_dict(extra_env)
        self.assertEqual(
            {
                "db_host": db_host,
                "db_port": db_port,
                "db_user": db_user,
                "db_password": db_password,
                "database": db_name,
            },
            dict_args,
        )

    def test_env_to_no_section(self):
        unknown_section = entry.env_to_section()
        self.assertFalse(unknown_section)
        unknown_section = entry.env_to_section(api.EnvConfigSection, {})
        self.assertFalse(unknown_section)

    def test_env_to_section(self):
        db_host = "my-host.com"
        db_name = "my-db"
        db_port = 5253
        db_user = "my-user"
        db_password = "py-password"
        extra_env = {
            "POSTGRESQL_ADDON_DB": db_name,
            "POSTGRESQL_ADDON_HOST": db_host,
            "POSTGRESQL_ADDON_PORT": db_port,
            "POSTGRESQL_ADDON_USER": db_user,
            "POSTGRESQL_ADDON_PASSWORD": db_password,
            "LONGPOLLING_PORT": 8072,
        }
        result_section = entry.env_to_section(DatabaseOdooConfigSection, extra_env)
        self.assertEqual(db_host, result_section.host)
        self.assertEqual(db_name, result_section.name)
        self.assertEqual(db_port, result_section.port)
        self.assertEqual(db_user, result_section.user)
        self.assertEqual(db_password, result_section.password)
        with self.assertRaises(AttributeError):
            result_section.longpolling_port

    def test_empty_env(self):
        result_section = entry.env_to_section(DatabaseOdooConfigSection)
        self.assertIsNone(result_section.host)
        self.assertIsNone(result_section.name)
        self.assertFalse(result_section.port)
        self.assertIsNone(result_section.user)
        self.assertIsNone(result_section.password)
        dict_args = entry.env_to_dict()
        self.assertFalse(dict_args)



# -*- coding: utf8 -*-
import unittest
from typing import List, Type

from src.odoo_env_config import api, cli
from src.odoo_env_config.cli import _load_profiles


class TestCli(unittest.TestCase):
    def test_cli1(self):
        parser = cli.get_odoo_cmd_parser()
        ns = parser.parse_args(
            [
                "--profile=/tmp/profile1.env",
                "--profile=/tmp/profile2.env",
                "--dest=/tmp/dest.ini",
            ]
        )
        self.assertEqual(ns.config_dest, "/tmp/dest.ini")
        self.assertIn("/tmp/profile1.env", ns.profiles)
        self.assertIn("/tmp/profile2.env", ns.profiles)

    def test_cli_profiles(self):
        parser = cli.get_odoo_cmd_parser()
        ns = parser.parse_args(["--profiles=/tmp/profile1.env,/tmp/profile2.env"])
        self.assertIn("/tmp/profile1.env", ns.profiles)
        self.assertIn("/tmp/profile2.env", ns.profiles)

    def test_cli_profiles_ending_comma(self):
        parser = cli.get_odoo_cmd_parser()
        ns = parser.parse_args(["--profiles=/tmp/profile1.env,/tmp/profile2.env,"])
        self.assertIn("/tmp/profile1.env", ns.profiles)
        self.assertIn("/tmp/profile2.env", ns.profiles)
        self.assertEqual(len(ns.profiles), 2)

    def test_cli_profiles_many_comma(self):
        parser = cli.get_odoo_cmd_parser()
        ns = parser.parse_args(["--profiles=/tmp/profile1.env,,/tmp/profile2.env"])
        self.assertIn("/tmp/profile1.env", ns.profiles)
        self.assertIn("/tmp/profile2.env", ns.profiles)
        self.assertEqual(len(ns.profiles), 2)

    def test_cli_profiles_no_comma(self):
        parser = cli.get_odoo_cmd_parser()
        ns = parser.parse_args(["--profiles=/tmp/profile1.env,"])
        self.assertIn("/tmp/profile1.env", ns.profiles)
        self.assertEqual(len(ns.profiles), 1)

    def test_cli_profiles_prefix_comma(self):
        parser = cli.get_odoo_cmd_parser()
        ns = parser.parse_args(["--profiles=,/tmp/profile1.env,/tmp/profile2.env"])
        self.assertIn("/tmp/profile1.env", ns.profiles)
        self.assertIn("/tmp/profile2.env", ns.profiles)
        self.assertEqual(len(ns.profiles), 2)

    def test_cli_profiles_both_options(self):
        parser = cli.get_odoo_cmd_parser()
        ns = parser.parse_args(
            [
                "--profile=/tmp/profile3.env",
                "--profiles=/tmp/profile1.env,/tmp/profile2.env",
            ]
        )
        self.assertIn("/tmp/profile1.env", ns.profiles)
        self.assertIn("/tmp/profile2.env", ns.profiles)
        self.assertIn("/tmp/profile3.env", ns.profiles)
        self.assertEqual(len(ns.profiles), 3)

    def test_cli_profiles_empty(self):
        parser = cli.get_odoo_cmd_parser()
        ns = parser.parse_args(["--profiles="])
        self.assertEqual(len(ns.profiles), 0)

    def test_load_profiles_without_profile(self):
        extra_env = _load_profiles([])
        self.assertEqual(extra_env, api.Env())


class _FakeOdooConfig(api.OdooConfig):
    def __init__(self):
        super(_FakeOdooConfig, self).__init__()
        self.args_parse_config: List[List[str]] = []
        self.save_is_called_nb: int = 0
        self.misc = {}
        self.options = {}

    def get(self, key, default=None):
        return self.options.get(key, default)

    def pop(self, key, default=None):
        return self.options.pop(key, default)

    def get_misc(self, sect, key, default=None):
        return self.misc.get(sect, {}).get(key, default)

    def __setitem__(self, key, value):
        self.options[key] = value

    def __getitem__(self, key):
        return self.options[key]

    def _parse_config(self, args: List[str]):
        print(args)
        self.args_parse_config.append(args)

    def save(self):
        print("Save")
        self.save_is_called_nb += 1


class FakeOdoo:  # noqa
    def __init__(self):
        self.tools = FakeOdoo._Tools()

    class cli:  # noqa
        class Command:
            sub_command: Type = None

            def __init_subclass__(cls, **kwargs):
                FakeOdoo.cli.Command.sub_command = cls

    class _Tools:  # noqa
        def __init__(self):
            self.config = _FakeOdooConfig()


class TestOdooCommand(unittest.TestCase):
    def test_create(self):
        """
        Assert the class [cli.OdooCommand][cli.OdooCommand] is correct to make a new odoo command

        1. The name is env2config`
        2. Have 1 function names 'run'
        3. Is subclass of odoo.cli.Command in the test case `_FakeOdooCommand`

        """
        fake_odoo = FakeOdoo()
        self.assertFalse(fake_odoo.cli.Command.sub_command)
        command = cli.OdooCommand(fake_odoo)
        self.assertEqual("env2config", cli.OdooCommand.command_name())

        self.assertTrue(fake_odoo.cli.Command.sub_command)
        clazz = fake_odoo.cli.Command.sub_command

        # Assert Class is created has expected
        self.assertIn(fake_odoo.cli.Command, clazz.mro())
        self.assertEqual(cli.OdooCommand.command_name(), clazz.__name__)

        # Assert Instance have attribute has expected
        inst_clazz = clazz()
        self.assertTrue(hasattr(inst_clazz, "run"))
        self.assertEqual(getattr(inst_clazz, "run"), command.run_command)

    def test_runcommand1(self):
        """
        Assert the `run` function of the dynamic odoo.cli.Command call
        1. the save function of the config
        2. split the env parse in 2 step (env args then odoo args)
        """
        fake_odoo = FakeOdoo()
        command = cli.OdooCommand(fake_odoo)
        parser_args = [
            "--profile=/tmp/profile1.env",
            "--profile=/tmp/profile2.env",
            "--dest=/tmp/dest.ini",
        ]
        odoo_args = ["-d", "my_database"]
        command.run_command(parser_args + odoo_args)
        odoo_args_res = fake_odoo.tools.config.args_parse_config
        self.assertEqual(fake_odoo.tools.config.save_is_called_nb, 1)
        self.assertEqual(len(odoo_args_res), 2)
        self.assertFalse(
            odoo_args_res[0], "Assert False because the file in `--profile` don't exist"
        )
        self.assertEqual(
            odoo_args_res[1], odoo_args, "Assert extra odoo args are keeped"
        )



import warnings
from collections import OrderedDict
from typing import Any, Dict, Optional, Set, Union

BOOL_ACCEPTED_VALUE: Set[str] = {str(1), str(0), str(True), str(False), "Yes", "No"}
BOOL_ACCEPTED_VALUE |= {a[0] for a in BOOL_ACCEPTED_VALUE}
BOOL_ACCEPTED_VALUE |= {a.lower() for a in BOOL_ACCEPTED_VALUE}
_FLOAT_PART = 2


def to_bool(anything: Union[Any, None]) -> bool:
    """
    Convert `anything` to a [bool][bool] according to [os.environ][os.environ] most found _True_ value.
    Args:
        anything: The value to convert
    Returns:
        `True` if `anything` in `True`, `"True"` `"1"`  otherwise `False`
    Notes:
        If the value is a string, then this is case **insensitive**
    Examples:
        >>> is_true("")
        False
        >>> is_true("True")
        True
        >>> is_true("TRUE")
        True
        >>> is_true("true")
        True
        >>> is_true("trUE")
        True
        >>> is_true("0")
        False
        >>> is_true("1")
        True
        >>> is_true([1, 2])
        False
        >>> is_true({"key": "value"})
        False
        >>> is_true((1, 2, 3))
        False

    """
    if not anything or not isinstance(anything, (str, bool, int, float)):
        return False
    return (
        bool(anything)
        and (str(anything).isdigit() and bool(int(anything)))
        or (str(anything).capitalize() == str(True))
        or False
    )


def is_boolean(anything: Union[Any, None]) -> bool:
    """
    Return `True` if the value is a boolean value.
    Args:
        anything: The value to test
    Returns:
        `True` if `anything` can be considered as a boolean value
    Notes:
        If the value is a string, then this is case **insensitive**
    Examples:
        >>> is_boolean("") or is_boolean(" ") or is_boolean(None)
        False
        >>> is_boolean("True") and is_boolean("true") and is_boolean("TrUe") and is_boolean("TRUE")
        True
        >>> is_boolean("False") and is_boolean("false") and is_boolean("FalSe") and is_boolean("FALSE")
        True
        >>> is_boolean("Yes") and is_boolean("Y") and is_boolean("y") and is_boolean("yes") and is_boolean("yES")
        True
        >>> is_boolean("No") and is_boolean("NO") and is_boolean("no") and is_boolean("n") and is_boolean("N")
        True
        >>> is_boolean("T") and is_boolean("t") and is_boolean("F") and is_boolean("F")
        True
        >>> is_boolean(1) and is_boolean(1) and is_boolean(0) and is_boolean("1") and is_boolean("0")
        True
        >>> is_boolean(2) or is_boolean(-1)
        False
        >>> is_boolean("Vrai") or is_boolean("Faux") or is_boolean("FauX") or is_boolean("VRai")
        False
        >>> is_boolean("V") or is_boolean("v")
        False
        >>> is_boolean(True) and is_boolean(False)
        True
    """
    return str(anything).lower() in BOOL_ACCEPTED_VALUE


def is_true(anything: Union[Any, None]) -> bool:
    """
    Deprecated, the name is confusing, with the new function `is_bool`
     Args:
        anything: The value to convert
    Returns:
        `True` if `anything` in `True`, `"True"` `"1"`  otherwise `False`
    Notes:
        If the value is a string, then this is case **insensitive**
        Any value that is not converted to True will be False
    """
    warnings.warn("Use to_bool instead", category=DeprecationWarning, stacklevel=2)
    return to_bool(anything)


def to_int(anything: Union[Any, None]) -> int:
    """
    Convert `anything` to a [int][int] according to [os.environ][os.environ] most found _int_ value.
    Args:
        anything: The value to convert
    Returns:
        the value as `int`, otherwise `0`
    Notes:
        If a float is given, then a hard convert to [int][int] is done. See Examples.
    Examples:
        >>> to_int("0") == 0
        True
        >>> to_int("0.0") == 0
        True
        >>> to_int("0.5") == 0
        True
        >>> to_int("0.5") == 0
        True
        >>> to_int(True) == 0
        True
        >>> to_int("True") == 0
        True
        >>> to_int([1, 2])
        0
        >>> to_int({"key": "value"})
        0
        >>> to_int((1, 2, 3))
        0
        >>> to_int(10)
        10
        >>> to_int("10")
        10

    """
    if not isinstance(anything, (str, int, float)) or not anything:
        return 0
    if isinstance(anything, bool):
        return 0
    if isinstance(anything, str):
        if anything.isdigit():
            return int(anything)
        float_parts = anything.split(".")
        if len(float_parts) == _FLOAT_PART and float_parts[0].isdigit() and float_parts[1].isdigit():
            return int(float(anything))
        return 0
    return int(anything)


def is_number(anything: Union[Any, None]) -> bool:
    """
    Convert `anything` to a [int][int] according to [os.environ][os.environ] most found _int_ value.
    Args:
        anything: The value to convert
    Returns:
        the value as `int`, otherwise `0`
    Notes:
        If a float is given, then a hard convert to [int][int] is done. See Examples.
    Examples:
        >>> is_number("0")
        True
        >>> is_number(10)
        True
        >>> is_number("10")
        True
        >>> is_number("0.0")
        True
        >>> is_number("0.5")
        True
        >>> is_number("-0.5")
        True
        >>> is_number(-0.5)
        True
        >>> is_number(True)
        False
        >>> is_number("True")
        False
        >>> is_number([1, 2])
        False
        >>> is_number({"key": "value"})
        False
        >>> is_number((1, 2, 3))
        False
        >>> is_number(None)
        False

    """
    if not isinstance(anything, (str, int, float)) or not anything:
        return False
    if isinstance(anything, bool):
        return False
    try:
        float(anything)
        return True
    except ValueError:
        return False


SUPPORTED_TRUE_VALUE = [True, str(True), str(True).upper(), str(True).lower()]
SUPPORTED_FALSE_VALUE = [False, str(False), str(False).upper(), str(False).lower()]


def is_bool(anything: Union[Any, None]) -> bool:
    """
    Return True if anythong is a valid bool value, True or False, as string or not.
    int, float or any other type are not valid bool
    Args:
        anything: The value to check
    Returns:
        `True` if `anything` in SUPPORTED_[TRUE | FALSE]_VALUE
    Notes:
        If the value is a string, then this is case **insensitive**
    Examples:
        >>> is_bool("")
        False
        >>> is_bool("True")
        True
        >>> is_bool("TRUE")
        True
        >>> is_bool("true")
        True
        >>> is_bool("trUE")
        True
        >>> is_bool(True)
        True
        >>> is_bool("False")
        True
        >>> is_bool("FALSE")
        True
        >>> is_bool("false")
        True
        >>> is_bool("FALse")
        True
        >>> is_bool(False)
        True
        >>> is_bool("0")
        False
        >>> is_bool("1")
        False
        >>> is_bool([1, 2])
        False
        >>> is_bool({"key": "value"})
        False
        >>> is_bool((1, 2, 3))
        False

    """
    return str(anything).upper() in SUPPORTED_TRUE_VALUE + SUPPORTED_FALSE_VALUE


def add_dash(value: str) -> str:
    """
    Return the value with 1 or 2 dash `-` as prefix
    If the value lenght is > 2 the 2 `-` is added as prefix else only one
    if the value is already prefixed, no dash added
    Args:
        value: The value, can contains `=`, in this case only the first part is take in the lenght of the option
    Returns:
        the value with dash
    Examples:
        >>> add_dash("my_option")
        '--my_option'
        >>> add_dash("k=value")
        '-k=value'
        >>> add_dash("key=value")
        '--key=value'
        >>> add_dash("--key=value")
        '--key=value'
        >>> add_dash("-o")
        '-o'
        >>> add_dash("i")
        '-i'
        >>> add_dash("-u=base")
        '-u=base'
        >>> add_dash(None)  # type check warning but ok
        ''
        >>> add_dash("")
        ''

    """
    if not value:
        return ""
    if not value.startswith("-"):
        dash = "-" * min(len(value.split("=", maxsplit=1)[0]), 2)
        return f"{dash}{value}"
    return value


def get_value(
    env_vars: Dict[str, str],
    *keys: str,
    default: str = None,
    if_all_falsy_return_none: bool = True,
) -> Optional[str]:
    """
    Allow to get the value from multiplke key inside a dict
    Usefull to search a value where the key can be multiple in [os.environ][os.environ]
    Args:
        env_vars: The dict to find in
        keys: all the values
        default: The default value if not any key found
        if_all_falsy_return_none: Force return `None` if the value is a `False` value

    Returns:
        if none_if_false==True : The first not falsy value found from keys (in keys orders) or None
        or default if keys do not exist
        if none_if_false==False : The first not falsy value found from keys (in keys orders) or
        the first falsy value found from keys or default if keys do not exist
    Examples:
        >>> get_value({"k1": "v1", "k2": "v2"}, "k1")
        'v1'
        >>> get_value({"k1": "v1", "k2": "v2"}, "k2")
        'v2'
        >>> get_value({"k1": "v1", "k2": "v2"}, "k2", "k1")
        'v2'
        >>> get_value({"k1": "v1", "k2": "v2"}, "k1", "k2")
        'v1'
        >>> get_value({"k1": "v1", "k2": "v2"}, "k3", "k2")
        'v2'
        >>> get_value(
        ...     {"k1": "v1", "k2": "v2"},
        ...     "k3",
        ... ) is None
        True
        >>> get_value({"k1": "v1", "k2": "v2"}, "k3", default="v3")
        'v3'
        >>> get_value({"k1": "", "k2": "v2"}, "k1", "k2", if_all_falsy_return_none=True)
        'v2'
        >>> get_value({"k1": "", "k2": "v2"}, "k1", if_all_falsy_return_none=False)
        ''
        >>> get_value({"k1": "", "k2": "v2"}, "k1", if_all_falsy_return_none=True) is None
        True
        >>> get_value({"k1": "", "k2": "v2"}, "k1", default="v1", if_all_falsy_return_none=False)
        ''
        >>> get_value({}, "k1", default="v1")
        'v1'
    """
    res = None
    last_falsy_value = default
    for key in keys:
        res = env_vars.get(key)
        if res:
            return res
        elif res is not None:
            last_falsy_value = res

    if if_all_falsy_return_none and (res is not None):
        return None

    return last_falsy_value


class DictUtil:
    """
    Utility class to manipulate [Dict][typing.Dict]
    """

    @staticmethod
    def clean_dict(values: Dict[str, Any]) -> Dict[str, str]:
        """
        Remove `False` value, `None` value
        Flat Dict value witha recursive call
        And convert List to `str` by comma joining value
        Args:
            values: A dict to clean and flat value

        Returns:
            A new dict without dirty value and None key value
        """
        new_values = OrderedDict()
        for key, value in values.items():
            if isinstance(value, dict):
                value = DictUtil.clean_dict(value)
            elif isinstance(value, (list, tuple, set)):
                value = ",".join([str(x) for x in value]) or ""
            if value is not False and value is not None and not isinstance(value, dict):
                new_values[key] = str(value)
        return new_values

    @staticmethod
    def clean_none_env_value(dict_value: Dict[str, Any]) -> Dict[str, Any]:
        """
        Like the name say, remove all the key with a `None` value
        Args:
            dict_value: A dict to clean

        Returns:
            A new Dict without the key where the value was `None`
        """
        result = OrderedDict()
        for key, value in dict_value.items():
            if value is not None:
                result[key] = value
        return result



from collections import OrderedDict

from typing_extensions import Self

from .. import api, utils


class ServerWideModuleConfigSection(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.server_wide_modules = []

    def init(self, curr_env: api.Env) -> Self:
        str_server_wide_modules = curr_env.get("SERVER_WIDE_MODULES")
        server_wide_modules = (
            str_server_wide_modules and list(OrderedDict().fromkeys(str_server_wide_modules.split(",")).keys()) or []
        )
        for suffix_key, value in curr_env.get_start_with("LOAD_"):
            if utils.is_number(value) or utils.is_bool(value):
                if utils.to_bool(value):
                    server_wide_modules.append(suffix_key.lower())
            else:
                server_wide_modules.append(value.lower())
        self.server_wide_modules = server_wide_modules
        return self

    def to_values(self) -> api.OdooCliFlag:
        return api.OdooCliFlag().set("load", self.server_wide_modules)



from typing_extensions import Self

from .. import api


class S3Section(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.access_key = False
        self.secret = False
        self.region = False
        self.host = False
        self.bucket_name = False
        self.secure = False
        self.sub_dir = False

    def init(self, curr_env: api.Env) -> Self:
        self.access_key = curr_env.get("S3_FILESTORE_ACCESS_KEY")
        self.secret = curr_env.get("S3_FILESTORE_SECRET_KEY")
        self.region = curr_env.get("S3_FILESTORE_REGION")
        self.host = curr_env.get("S3_FILESTORE_HOST")
        self.bucket_name = curr_env.get("S3_FILESTORE_BUCKET")
        self.secure = curr_env.get_bool("S3_SECURE", default=True)
        self.sub_dir = curr_env.get_bool("S3_FILESTORE_SUB_DIR", default=False)
        return self

    def write_to_config(self, config: api.OdooConfig):
        if not self.host:
            return
        config.misc["odoo_s3_filestore"] = {
            "s3_access_key": self.access_key,
            "s3_secret": self.secret,
            "s3_region": self.region,
            "s3_host": self.host,
            "s3_bucket_name": self.bucket_name,
            "s3_secure": self.secure,
            "sub_dir_by_dbname": self.sub_dir,
        }



from typing_extensions import Self

from .. import api


class UpdateInstallSection(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.update = []
        self.install = []

    def init(self, curr_env: api.Env) -> Self:
        self.update = curr_env.get_list("UPDATE")
        self.install = curr_env.get_list("INSTALL")
        return self

    def to_values(self) -> api.OdooCliFlag:
        flags = api.OdooCliFlag()
        flags.set("update", self.update and ",".join(self.update))
        flags.set("init", self.install and ",".join(self.install))
        return flags



import unittest

from src.odoo_env_config.api import Env
from src.odoo_env_config.section import S3Section


class TestLogSection(unittest.TestCase):
    def test_default(self):
        conf = S3Section()
        self.assertFalse(conf.access_key)
        self.assertFalse(conf.secret)
        self.assertFalse(conf.region)
        self.assertFalse(conf.host)
        self.assertFalse(conf.bucket_name)
        self.assertFalse(conf.secure)

    def test_not_implmented(self):
        with self.assertRaises(NotImplementedError):
            S3Section().to_values()

    def test_values(self):
        conf = S3Section().init(
            Env(
                {
                    "S3_FILESTORE_ACCESS_KEY": "s3_access_value",
                    "S3_FILESTORE_SECRET_KEY": "s3_secret_value",
                    "S3_FILESTORE_BUCKET": "s3_bucket_value",
                    "S3_FILESTORE_HOST": "s3_host_value",
                    "S3_FILESTORE_REGION": "s3_region_value",
                    "S3_SECURE": "True",
                }
            )
        )
        self.assertEqual("s3_access_value", conf.access_key)
        self.assertEqual("s3_secret_value", conf.secret)
        self.assertEqual("s3_bucket_value", conf.bucket_name)
        self.assertEqual("s3_host_value", conf.host)
        self.assertEqual("s3_region_value", conf.region)
        self.assertTrue(conf.secure)

    def test_subdir(self):
        conf = S3Section().init(
            Env(
                {
                    "S3_FILESTORE_ACCESS_KEY": "s3_access_value",
                    "S3_FILESTORE_SECRET_KEY": "s3_secret_value",
                    "S3_FILESTORE_BUCKET": "s3_bucket_value",
                    "S3_FILESTORE_HOST": "s3_host_value",
                    "S3_FILESTORE_SUB_DIR": "True",
                    "S3_SECURE": "False",
                }
            )
        )
        self.assertEqual("s3_access_value", conf.access_key)
        self.assertEqual("s3_secret_value", conf.secret)
        self.assertEqual("s3_bucket_value", conf.bucket_name)
        self.assertEqual("s3_host_value", conf.host)
        self.assertFalse(conf.region)
        self.assertFalse(conf.secure)
        self.assertTrue(conf.sub_dir)



import unittest

from tests import _decorators

from ._helpers import assertParser, create_config


class BaseCase:
    class OdooVersionTest(unittest.TestCase):
        @classmethod
        def setUpClass(cls):
            cls.template_name = "template0"

        def test_db(self):
            parser = create_config()
            self.assertFalse(parser.getboolean("options", "db_host"))
            self.assertEqual(64, parser.getint("options", "db_maxconn"))
            self.assertFalse(parser.getboolean("options", "db_name"))
            self.assertFalse(parser.getboolean("options", "db_password"))
            self.assertFalse(parser.getboolean("options", "db_port"))
            self.assertEqual("prefer", parser.get("options", "db_sslmode"))
            self.assertEqual(self.template_name, parser.get("options", "db_template"))
            self.assertFalse(parser.getboolean("options", "db_user"))
            self.assertEqual("", parser.get("options", "dbfilter"))

        def test_db_default(self):
            parser = create_config()
            assertParser(
                self,
                parser,
                {
                    "db_host": False,
                    "db_maxconn": 64,
                    "db_name": False,
                    "db_password": False,
                    "db_port": False,
                    "db_sslmode": "prefer",
                    "db_template": self.template_name,
                    "db_user": False,
                    "dbfilter": "",
                },
            )

        def test_db_default15(self):
            parser = create_config()
            assertParser(
                self,
                parser,
                {
                    "db_host": False,
                    "db_maxconn": 64,
                    "db_name": False,
                    "db_password": False,
                    "db_port": False,
                    "db_sslmode": "prefer",
                    "db_template": self.template_name,
                    "db_user": False,
                    "dbfilter": "",
                },
            )

        def test_db_profile_clever(self):
            parser = create_config(["test_db_clever"])
            assertParser(
                self,
                parser,
                {
                    "db_host": "my_db_host",
                    "db_maxconn": 64,
                    "db_name": "my_db_name",
                    "db_password": "my_db_password",
                    "db_port": 1234,
                    "db_sslmode": "prefer",
                    "db_template": self.template_name,
                    "db_user": "my_db_user",
                    "dbfilter": "",
                },
            )

        def test_db_profile_clever_direct(self):
            parser = create_config(["test_db_clever_direct"])
            assertParser(
                self,
                parser,
                {
                    "db_host": "my_db_host_direct",
                    "db_maxconn": 64,
                    "db_name": "my_db_name_direct",
                    "db_password": "my_db_password_direct",
                    "db_port": 4567,
                    "db_sslmode": "prefer",
                    "db_template": self.template_name,
                    "db_user": "my_db_user_direct",
                    "dbfilter": "",
                },
            )

        def test_db_profile_clever_full_both(self):
            parser = create_config(["test_db_clever", "test_db_clever_direct"])
            assertParser(
                self,
                parser,
                {
                    "db_host": "my_db_host_direct",
                    "db_maxconn": 64,
                    "db_name": "my_db_name_direct",
                    "db_password": "my_db_password_direct",
                    "db_port": 4567,
                    "db_sslmode": "prefer",
                    "db_template": self.template_name,
                    "db_user": "my_db_user_direct",
                    "dbfilter": "",
                },
            )

        def test_runbot_classic_config(self):
            parser = create_config(["runbot_classic_before"])
            assertParser(
                self,
                parser,
                {
                    "db_host": "database",
                    "db_maxconn": 64,
                    "db_name": "databasehbkw",
                    "db_password": "ltniqmdrwo",
                    "db_port": 5432,
                    "db_sslmode": "prefer",
                    "db_template": self.template_name,
                    "db_user": "ddjwl",
                    "log_level": "debug",
                    "without_demo": "False",
                    "test_enable": "False",
                },
            )
            self.assertNotEqual("admin", parser.get("options", "admin_passwd"))
            self.assertTrue(
                parser.get("options", "admin_passwd").startswith("$pbkdf2-sha512$"),
                parser.get("options", "admin_passwd"),
            )


@_decorators.SkipUnless.env_odoo11
class BaseOdooVersionTest11(BaseCase.OdooVersionTest):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.template_name = "template1"


@_decorators.SkipUnless.odoo_more11
class BaseOdooVersionTest12AndMore(BaseCase.OdooVersionTest): ...



import unittest

from src.odoo_env_config.api import Env
from src.odoo_env_config.section import MiscSection


class TestMiscSection(unittest.TestCase):
    def test_default(self):
        conf = MiscSection()
        self.assertFalse(conf.unaccent)
        self.assertEqual([], conf.without_demo)
        self.assertFalse(conf.stop_after_init)
        self.assertFalse(conf.save_config_file)

        flags = conf.to_values()
        self.assertFalse(flags)

    def test_global(self):
        conf = MiscSection().init(
            Env(
                {
                    "UNACCENT": str(True),
                    "WITHOUT_DEMO": "all,account",
                    "STOP_AFTER_INIT": str(True),
                    "SAVE_CONFIG_FILE": str(True),
                    "DATA_DIR": "data",
                }
            )
        )
        self.assertTrue(conf.unaccent)
        self.assertListEqual(["all", "account"], conf.without_demo)
        self.assertTrue(conf.stop_after_init)
        self.assertTrue(conf.save_config_file)

        flags = conf.to_values()
        self.assertIn("unaccent", flags)
        self.assertTrue(flags["unaccent"])

        self.assertIn("without-demo", flags)
        self.assertEqual("all,account", flags["without-demo"])

        self.assertIn("save", flags)
        self.assertTrue(flags["save"])

        self.assertIn("stop-after-init", flags)
        self.assertTrue(flags["stop-after-init"])

        self.assertIn("data-dir", flags)
        self.assertEqual(flags["data-dir"], "data")

    def test_datadir_sub_ODOO_PATH(self):
        conf = MiscSection().init(
            Env(
                {
                    "ODOO_PATH": "/odoo",
                    "DATA_DIR": "data",
                }
            )
        )
        self.assertEqual(conf.data_dir, "/odoo/data")
        flags = conf.to_values()
        self.assertIn("data-dir", flags)
        self.assertEqual(flags["data-dir"], "/odoo/data")

    def test_without_demo_FALSE(self):
        conf = MiscSection().init(
            Env(
                {
                    "WITHOUT_DEMO": "False",
                }
            )
        )
        self.assertFalse(conf.without_demo)
        flags = conf.to_values()
        self.assertNotIn("without-demo", flags)

    def test_without_demo_True(self):
        """
        Env WITHOUT_DEMO=True handle as --without-demo=all
        Returns:

        """
        conf = MiscSection().init(
            Env(
                {
                    "WITHOUT_DEMO": "True",
                }
            )
        )
        self.assertListEqual(["all"], conf.without_demo)
        flags = conf.to_values()
        self.assertIn("without-demo", flags)
        self.assertEqual("all", flags["without-demo"])



import unittest

from tests import _decorators

from ._helpers import assertParser, create_config


@_decorators.SkipUnless.env_odoo
class TestMiscS3(unittest.TestCase):
    def test_filestore_s3(self):
        """
        Assert module `odoo_filestore_s3` is enable.
        Returns:

        """
        parser = create_config(["filestore_s3"])
        assertParser(
            self,
            parser,
            {
                "server_wide_modules": ["odoo_filestore_s3"],
            },
        )
        assertParser(
            self,
            parser,
            {
                "s3_access_key": "s3_filestore_access_key",
                "s3_secret": "s3_filestore_secret_key",
                "s3_region": "s3_filestore_region",
                "s3_host": "s3_filestore_host",
                "s3_bucket_name": "s3_filestore_bucket",
                "s3_secure": True,
                "sub_dir_by_dbname": False,
            },
            section="odoo_s3_filestore",
        )

    def test_filestore_s3_cellar(self):
        parser = create_config(["filestore_s3_cellar"])
        assertParser(
            self,
            parser,
            {
                "server_wide_modules": ["odoo_filestore_s3"],
            },
        )
        assertParser(
            self,
            parser,
            {
                "s3_access_key": "qsdlkqjsdlkq",
                "s3_secret": "qsdklqjsdlqksdj",
                "s3_host": "cellar-c2.services.clever-cloud.com",
                "s3_bucket_name": "cellar_s3_filestore_bucket",
                "s3_secure": True,
                "sub_dir_by_dbname": True,
            },
            section="odoo_s3_filestore",
        )



from __future__ import annotations

import enum
import logging
import os
from collections import OrderedDict

from typing_extensions import (
    Any,
    Dict,
    List,
    Protocol,
    Self,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from . import utils

_logger = logging.getLogger(__name__)


def monkey_patch(cls):
    """Return a method decorator to monkey-patch the given class."""

    def decorate(func):
        name = func.__name__
        func.super = getattr(cls, name, None)
        setattr(cls, name, func)
        return func

    return decorate


class ServerUseCase(enum.Enum):
    WORKER = "WORKER"
    TESTS = "TESTS"
    THREADED_MODE = "THREADED_MODE"
    ONLY_HTTP = "ONLY_HTTP"
    ONLY_JOB_RUNNER = "ONLY_JOB_RUNNER"
    ONLY_JOB_WORKER = "ONLY_JOB_WORKER"
    ONLY_CRON = "ONLY_CRON"


ET = TypeVar("ET", bound=enum.Enum)

@enum.unique
class OdooVersion(enum.IntEnum):
    NO_VERSION=0 # Special value
    V11 = 11
    V12 = 12
    V13 = 13
    V14 = 14
    V15 = 15
    V16 = 16
    V17 = 17
    V18 = 18


class Env(dict, Dict[str, str]):
    def copy(self) -> Self:
        return Env(self)

    @property
    def odoo_version(self) -> int:
        """
        Returns:
            `env:ODOO_VERSION` as [int][int]
        """
        return self.odoo_version_type.value

    @property
    def odoo_version_type(self) -> OdooVersion:
        """
        Returns:
            `env:ODOO_VERSION` as OdooVersion
        """
        int_version = self.get_int("ODOO_VERSION")
        return OdooVersion(min(int_version, OdooVersion.V18.value))

    @property
    def main_instance(self) -> bool:
        """
        Returns:
            `True` if `instance_number` `==` `0`
        """
        return self.instance_number == 0

    @property
    def instance_number(self) -> int:
        """
        Returns:
            `env:INSTANCE_NUMBER`
        """
        return self.get_int("INSTANCE_NUMBER")

    @property
    def odoo_bin(self) -> str:
        """
        Returns:
            The `env:ODOO_BIN` or `/odoo/bin` path even if not exist
        """
        return self.get("ODOO_BIN", os.path.join("/odoo", "odoo-bin"))

    def mutate(self, *arg, **kwargs) -> Self:
        """
        Same as [dict.update][dict.update] but return `self`
        Returns:
            current self after the `update`
        """
        self.update(*arg, **kwargs)
        return self

    def get_bool(self, *keys: str, default: bool = False) -> bool:
        return utils.to_bool(self.gets(*keys, default=str(default)))

    def get_int(self, *keys: str, default: int = 0) -> int:
        return utils.to_int(self.gets(*keys, default=str(default)))

    def get_enum(self, key: str, enum_type: Type[ET], *, default: ET) -> ET:
        value = self.get(key)
        if not value or value not in enum_type.__members__:
            return default
        return enum_type[value]

    def is_boolean(self, *keys: str) -> bool:
        return utils.is_boolean(self.gets(*keys))

    def get_list(self, key: str, separator=",", allow_duplicate: bool = False) -> List[str]:
        value = self.get(key)
        if self.is_boolean(key) and not self.get_bool(key):
            return []
        if not value:
            return []
        if allow_duplicate:
            return [u.strip() for u in value.strip().split(separator)]
        res = OrderedDict()
        for value in value.strip().split(separator):
            res[value.strip()] = None
        return list(res.keys())

    def gets(self, *keys: str, default: str = None, none_if_false: bool = True) -> Union[str, None]:
        """
        Returns:
            The first not false value found from keys (in keys orders) or the default
        """
        return utils.get_value(self, *keys, default=default, if_all_falsy_return_none=none_if_false)

    def get_start_with(self, prefix) -> List[Tuple[str, str]]:
        result = {}
        for key, value in self.items():
            if key.startswith(prefix) and len(key) > len(prefix):
                result[key[len(prefix) :]] = value
        return cast(List[Tuple[str, str]], list(result.items()))


class OdooCliFlag(dict, Dict[str, Any]):
    def set(self, key: str, value: Any, force_set: bool = False) -> Self:
        if value or force_set:
            self[key] = value
        return self

    def set_all(self, values: Dict[str, Any], force_set: bool = False) -> Self:
        for key, value in values.items():
            self.set(key, value, force_set)
        return self


def dict_to_odoo_args(values: Dict[str, Any]) -> List[str]:
    result = []
    clean_values = utils.DictUtil.clean_none_env_value(values)
    clean_values = utils.DictUtil.clean_dict(clean_values)
    for key, value in clean_values.items():
        if not value:
            continue
        key = utils.add_dash(key)
        if value == str(True):
            result.append(key)
        else:
            result.append(f"{key}={value}")
    return result


class OdooConfig(Protocol):
    options: Dict[str, Any]
    misc: Dict[str, Dict[str, Any]]

    def get(self, key, default=None): ...

    def pop(self, key, default=None): ...

    def get_misc(self, sect, key, default=None): ...

    def __setitem__(self, key, value): ...

    def __getitem__(self, key): ...


class EnvConfigSection:
    """
    Base class for section
    A section is a part of the CLI of Odoo
    It takes an env and return the odoo cli flags
    """

    def __init__(self): ...

    def init(self, curr_env: Env) -> Self:
        return self

    def to_values(self) -> OdooCliFlag:
        raise NotImplementedError("Not implemented in this section {}".format(type(self).__name__))

    def write_to_config(self, config: OdooConfig):
        raise NotImplementedError()
